@@ -1,10 +1,10 @@
 #!/usr/bin/env python
-# pylint: disable=W0613, C0116
-# type: ignore[union-attr]
+# pylint: disable=C0116,W0613
 # This program is dedicated to the public domain under the CC0 license.
 
 """
-Basic example for a bot that uses inline keyboards.
+Basic example for a bot that uses inline keyboards. For an in-depth explanation, check out
+ https://git.io/JOmFw.
 """
 import logging
 
@@ -18,6 +18,7 @@ logger = logging.getLogger(__name__)
 
 
 def start(update: Update, context: CallbackContext) -> None:
+    """Sends a message with three inline buttons attached."""
     keyboard = [
         [
             InlineKeyboardButton("Option 1", callback_data='1'),
@@ -32,6 +33,7 @@ def start(update: Update, context: CallbackContext) -> None:
 
 
 def button(update: Update, context: CallbackContext) -> None:
+    """Parses the CallbackQuery and updates the message text."""
     query = update.callback_query
 
     # CallbackQueries need to be answered, even if no notification to the user is needed
@@ -42,10 +44,12 @@ def button(update: Update, context: CallbackContext) -> None:
 
 
 def help_command(update: Update, context: CallbackContext) -> None:
+    """Displays info on how to use the bot."""
     update.message.reply_text("Use /start to test this bot.")
 
 
-def main():
+def main() -> None:
+    """Run the bot."""
     # Create the Updater and pass it your bot's token.
     updater = Updater("TOKEN")
 
