@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -24,7 +24,18 @@ from .user import User
 from .files.chatphoto import ChatPhoto
 from .chat import Chat
 from .chatlocation import ChatLocation
-from .chatmember import ChatMember
+from .chatinvitelink import ChatInviteLink
+from .chatjoinrequest import ChatJoinRequest
+from .chatmember import (
+    ChatMember,
+    ChatMemberOwner,
+    ChatMemberAdministrator,
+    ChatMemberMember,
+    ChatMemberRestricted,
+    ChatMemberLeft,
+    ChatMemberBanned,
+)
+from .chatmemberupdated import ChatMemberUpdated
 from .chatpermissions import ChatPermissions
 from .files.photosize import PhotoSize
 from .files.audio import Audio
@@ -40,8 +51,8 @@ from .files.videonote import VideoNote
 from .chataction import ChatAction
 from .dice import Dice
 from .userprofilephotos import UserProfilePhotos
-from .keyboardbutton import KeyboardButton
 from .keyboardbuttonpolltype import KeyboardButtonPollType
+from .keyboardbutton import KeyboardButton
 from .replymarkup import ReplyMarkup
 from .replykeyboardmarkup import ReplyKeyboardMarkup
 from .replykeyboardremove import ReplyKeyboardRemove
@@ -54,6 +65,12 @@ from .messageentity import MessageEntity
 from .messageid import MessageId
 from .games.game import Game
 from .poll import Poll, PollOption, PollAnswer
+from .voicechat import (
+    VoiceChatStarted,
+    VoiceChatEnded,
+    VoiceChatParticipantsInvited,
+    VoiceChatScheduled,
+)
 from .loginurl import LoginUrl
 from .proximityalerttriggered import ProximityAlertTriggered
 from .games.callbackgame import CallbackGame
@@ -68,6 +85,7 @@ from .passport.encryptedpassportelement import EncryptedPassportElement
 from .passport.passportdata import PassportData
 from .inline.inlinekeyboardbutton import InlineKeyboardButton
 from .inline.inlinekeyboardmarkup import InlineKeyboardMarkup
+from .messageautodeletetimerchanged import MessageAutoDeleteTimerChanged
 from .message import Message
 from .callbackquery import CallbackQuery
 from .choseninlineresult import ChosenInlineResult
@@ -97,8 +115,9 @@ from .inline.inlinequeryresultgame import InlineQueryResultGame
 from .inline.inputtextmessagecontent import InputTextMessageContent
 from .inline.inputlocationmessagecontent import InputLocationMessageContent
 from .inline.inputvenuemessagecontent import InputVenueMessageContent
-from .inline.inputcontactmessagecontent import InputContactMessageContent
 from .payment.labeledprice import LabeledPrice
+from .inline.inputinvoicemessagecontent import InputInvoiceMessageContent
+from .inline.inputcontactmessagecontent import InputContactMessageContent
 from .payment.shippingoption import ShippingOption
 from .payment.precheckoutquery import PreCheckoutQuery
 from .payment.shippingquery import ShippingQuery
@@ -143,8 +162,18 @@ from .passport.credentials import (
     FileCredentials,
     TelegramDecryptionError,
 )
+from .botcommandscope import (
+    BotCommandScope,
+    BotCommandScopeDefault,
+    BotCommandScopeAllPrivateChats,
+    BotCommandScopeAllGroupChats,
+    BotCommandScopeAllChatAdministrators,
+    BotCommandScopeChat,
+    BotCommandScopeChatAdministrators,
+    BotCommandScopeChatMember,
+)
 from .bot import Bot
-from .version import __version__  # noqa: F401
+from .version import __version__, bot_api_version  # noqa: F401
 
 __author__ = 'devs@python-telegram-bot.org'
 
@@ -153,12 +182,29 @@ __all__ = (  # Keep this alphabetically ordered
     'Audio',
     'Bot',
     'BotCommand',
+    'BotCommandScope',
+    'BotCommandScopeAllChatAdministrators',
+    'BotCommandScopeAllGroupChats',
+    'BotCommandScopeAllPrivateChats',
+    'BotCommandScopeChat',
+    'BotCommandScopeChatAdministrators',
+    'BotCommandScopeChatMember',
+    'BotCommandScopeDefault',
     'CallbackGame',
     'CallbackQuery',
     'Chat',
     'ChatAction',
+    'ChatInviteLink',
+    'ChatJoinRequest',
     'ChatLocation',
     'ChatMember',
+    'ChatMemberOwner',
+    'ChatMemberAdministrator',
+    'ChatMemberMember',
+    'ChatMemberRestricted',
+    'ChatMemberLeft',
+    'ChatMemberBanned',
+    'ChatMemberUpdated',
     'ChatPermissions',
     'ChatPhoto',
     'ChosenInlineResult',
@@ -201,6 +247,7 @@ __all__ = (  # Keep this alphabetically ordered
     'InlineQueryResultVoice',
     'InputContactMessageContent',
     'InputFile',
+    'InputInvoiceMessageContent',
     'InputLocationMessageContent',
     'InputMedia',
     'InputMediaAnimation',
@@ -226,6 +273,7 @@ __all__ = (  # Keep this alphabetically ordered
     'MAX_MESSAGE_LENGTH',
     'MaskPosition',
     'Message',
+    'MessageAutoDeleteTimerChanged',
     'MessageEntity',
     'MessageId',
     'OrderInfo',
@@ -272,5 +320,9 @@ __all__ = (  # Keep this alphabetically ordered
     'Video',
     'VideoNote',
     'Voice',
+    'VoiceChatStarted',
+    'VoiceChatEnded',
+    'VoiceChatScheduled',
+    'VoiceChatParticipantsInvited',
     'WebhookInfo',
 )
