@@ -2,7 +2,7 @@
 # pylint: disable=R0903
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -18,9 +18,10 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains an object that represents a Telegram Bot Command."""
-from telegram import TelegramObject
 from typing import Any
 
+from telegram import TelegramObject
+
 
 class BotCommand(TelegramObject):
     """
@@ -29,16 +30,20 @@ class BotCommand(TelegramObject):
     Objects of this class are comparable in terms of equality. Two objects of this class are
     considered equal, if their :attr:`command` and :attr:`description` are equal.
 
+    Args:
+        command (:obj:`str`): Text of the command, 1-32 characters. Can contain only lowercase
+            English letters, digits and underscores.
+        description (:obj:`str`): Description of the command, 1-256 characters.
+
     Attributes:
         command (:obj:`str`): Text of the command.
         description (:obj:`str`): Description of the command.
 
-    Args:
-        command (:obj:`str`): Text of the command, 1-32 characters. Can contain only lowercase
-            English letters, digits and underscores.
-        description (:obj:`str`): Description of the command, 3-256 characters.
     """
-    def __init__(self, command: str, description: str, **kwargs: Any):
+
+    __slots__ = ('description', '_id_attrs', 'command')
+
+    def __init__(self, command: str, description: str, **_kwargs: Any):
         self.command = command
         self.description = description
 
