@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,6 +16,7 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
+# pylint: disable=C0115
 """This module contains an object that represents Telegram errors."""
 from typing import Tuple
 
@@ -31,13 +32,18 @@ def _lstrip_str(in_s: str, lstr: str) -> str:
 
     """
     if in_s.startswith(lstr):
-        res = in_s[len(lstr):]
+        res = in_s[len(lstr) :]
     else:
         res = in_s
     return res
 
 
 class TelegramError(Exception):
+    """Base class for Telegram errors."""
+
+    # Apparently the base class Exception already has __dict__ in it, so its not included here
+    __slots__ = ('message',)
+
     def __init__(self, message: str):
         super().__init__()
 
@@ -57,10 +63,16 @@ class TelegramError(Exception):
 
 
 class Unauthorized(TelegramError):
-    pass
+    """Raised when the bot has not enough rights to perform the requested action."""
+
+    __slots__ = ()
 
 
 class InvalidToken(TelegramError):
+    """Raised when the token is invalid."""
+
+    __slots__ = ()
+
     def __init__(self) -> None:
         super().__init__('Invalid token')
 
@@ -69,14 +81,22 @@ class InvalidToken(TelegramError):
 
 
 class NetworkError(TelegramError):
-    pass
+    """Base class for exceptions due to networking errors."""
+
+    __slots__ = ()
 
 
 class BadRequest(NetworkError):
-    pass
+    """Raised when Telegram could not process the request correctly."""
+
+    __slots__ = ()
 
 
 class TimedOut(NetworkError):
+    """Raised when a request took too long to finish."""
+
+    __slots__ = ()
+
     def __init__(self) -> None:
         super().__init__('Timed out')
 
@@ -86,13 +106,17 @@ class TimedOut(NetworkError):
 
 class ChatMigrated(TelegramError):
     """
+    Raised when the requested group chat migrated to supergroup and has a new chat id.
+
     Args:
         new_chat_id (:obj:`int`): The new chat id of the group.
 
     """
 
+    __slots__ = ('new_chat_id',)
+
     def __init__(self, new_chat_id: int):
-        super().__init__('Group migrated to supergroup. New chat id: {}'.format(new_chat_id))
+        super().__init__(f'Group migrated to supergroup. New chat id: {new_chat_id}')
         self.new_chat_id = new_chat_id
 
     def __reduce__(self) -> Tuple[type, Tuple[int]]:  # type: ignore[override]
@@ -101,13 +125,17 @@ class ChatMigrated(TelegramError):
 
 class RetryAfter(TelegramError):
     """
+    Raised when flood limits where exceeded.
+
     Args:
         retry_after (:obj:`int`): Time in seconds, after which the bot can retry the request.
 
     """
 
+    __slots__ = ('retry_after',)
+
     def __init__(self, retry_after: int):
-        super().__init__('Flood control exceeded. Retry in {} seconds'.format(float(retry_after)))
+        super().__init__(f'Flood control exceeded. Retry in {float(retry_after)} seconds')
         self.retry_after = float(retry_after)
 
     def __reduce__(self) -> Tuple[type, Tuple[float]]:  # type: ignore[override]
@@ -115,16 +143,9 @@ class RetryAfter(TelegramError):
 
 
 class Conflict(TelegramError):
-    """
-        Raised when a long poll or webhook conflicts with another one.
+    """Raised when a long poll or webhook conflicts with another one."""
 
-        Args:
-            msg (:obj:`str`): The message from telegrams server.
-
-    """
-
-    def __init__(self, msg: str):
-        super().__init__(msg)
+    __slots__ = ()
 
     def __reduce__(self) -> Tuple[type, Tuple[str]]:
         return self.__class__, (self.message,)
