@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -19,16 +19,32 @@
 # pylint: disable=R0201
 """This module contains the CallbackContext class."""
 from queue import Queue
-from typing import TYPE_CHECKING, Dict, List, Match, NoReturn, Optional, Tuple, Union
+from typing import (
+    TYPE_CHECKING,
+    Dict,
+    List,
+    Match,
+    NoReturn,
+    Optional,
+    Tuple,
+    Union,
+    Generic,
+    Type,
+    TypeVar,
+)
 
-from telegram import Update
+from telegram import Update, CallbackQuery
+from telegram.ext import ExtBot
+from telegram.ext.utils.types import UD, CD, BD
 
 if TYPE_CHECKING:
     from telegram import Bot
     from telegram.ext import Dispatcher, Job, JobQueue
 
+CC = TypeVar('CC', bound='CallbackContext')
 
-class CallbackContext:
+
+class CallbackContext(Generic[UD, CD, BD]):
     """
     This is a context object passed to the callback called by :class:`telegram.ext.Handler`
     or by the :class:`telegram.ext.Dispatcher` in an error handler added by
@@ -50,20 +66,10 @@ class CallbackContext:
          almost certainly execute the callbacks for an update out of order, and the attributes
          that you think you added will not be present.
 
+    Args:
+        dispatcher (:class:`telegram.ext.Dispatcher`): The dispatcher associated with this context.
+
     Attributes:
-        bot_data (:obj:`dict`): Optional. A dict that can be used to keep any data in. For each
-            update it will be the same ``dict``.
-        chat_data (:obj:`dict`): Optional. A dict that can be used to keep any data in. For each
-            update from the same chat id it will be the same ``dict``.
-
-            Warning:
-                When a group chat migrates to a supergroup, its chat id will change and the
-                ``chat_data`` needs to be transferred. For details see our `wiki page
-                <https://github.com/python-telegram-bot/python-telegram-bot/wiki/
-                Storing-user--and-chat-related-data#chat-migration>`_.
-
-        user_data (:obj:`dict`): Optional. A dict that can be used to keep any data in. For each
-            update from the same user it will be the same ``dict``.
         matches (List[:obj:`re match object`]): Optional. If the associated update originated from
             a regex-supported handler or had a :class:`Filters.regex`, this will contain a list of
             match objects for every pattern where ``re.search(pattern, string)`` returned a match.
@@ -86,6 +92,19 @@ class CallbackContext:
 
     """
 
+    __slots__ = (
+        '_dispatcher',
+        '_chat_id_and_data',
+        '_user_id_and_data',
+        'args',
+        'matches',
+        'error',
+        'job',
+        'async_args',
+        'async_kwargs',
+        '__dict__',
+    )
+
     def __init__(self, dispatcher: 'Dispatcher'):
         """
         Args:
@@ -96,9 +115,8 @@ class CallbackContext:
                 'CallbackContext should not be used with a non context aware ' 'dispatcher!'
             )
         self._dispatcher = dispatcher
-        self._bot_data = dispatcher.bot_data
-        self._chat_data: Optional[Dict[object, object]] = None
-        self._user_data: Optional[Dict[object, object]] = None
+        self._chat_id_and_data: Optional[Tuple[int, CD]] = None
+        self._user_id_and_data: Optional[Tuple[int, UD]] = None
         self.args: Optional[List[str]] = None
         self.matches: Optional[List[Match]] = None
         self.error: Optional[Exception] = None
@@ -112,8 +130,11 @@ class CallbackContext:
         return self._dispatcher
 
     @property
-    def bot_data(self) -> Dict:
-        return self._bot_data
+    def bot_data(self) -> BD:
+        """:obj:`dict`: Optional. A dict that can be used to keep any data in. For each
+        update it will be the same ``dict``.
+        """
+        return self.dispatcher.bot_data
 
     @bot_data.setter
     def bot_data(self, value: object) -> NoReturn:
@@ -122,8 +143,19 @@ class CallbackContext:
         )
 
     @property
-    def chat_data(self) -> Optional[Dict]:
-        return self._chat_data
+    def chat_data(self) -> Optional[CD]:
+        """:obj:`dict`: Optional. A dict that can be used to keep any data in. For each
+        update from the same chat id it will be the same ``dict``.
+
+        Warning:
+            When a group chat migrates to a supergroup, its chat id will change and the
+            ``chat_data`` needs to be transferred. For details see our `wiki page
+            <https://github.com/python-telegram-bot/python-telegram-bot/wiki/
+            Storing-bot,-user-and-chat-related-data#chat-migration>`_.
+        """
+        if self._chat_id_and_data:
+            return self._chat_id_and_data[1]
+        return None
 
     @chat_data.setter
     def chat_data(self, value: object) -> NoReturn:
@@ -132,8 +164,13 @@ class CallbackContext:
         )
 
     @property
-    def user_data(self) -> Optional[Dict]:
-        return self._user_data
+    def user_data(self) -> Optional[UD]:
+        """:obj:`dict`: Optional. A dict that can be used to keep any data in. For each
+        update from the same user it will be the same ``dict``.
+        """
+        if self._user_id_and_data:
+            return self._user_id_and_data[1]
+        return None
 
     @user_data.setter
     def user_data(self, value: object) -> NoReturn:
@@ -141,15 +178,82 @@ class CallbackContext:
             "You can not assign a new value to user_data, see https://git.io/Jt6ic"
         )
 
+    def refresh_data(self) -> None:
+        """If :attr:`dispatcher` uses persistence, calls
+        :meth:`telegram.ext.BasePersistence.refresh_bot_data` on :attr:`bot_data`,
+        :meth:`telegram.ext.BasePersistence.refresh_chat_data` on :attr:`chat_data` and
+        :meth:`telegram.ext.BasePersistence.refresh_user_data` on :attr:`user_data`, if
+        appropriate.
+
+        .. versionadded:: 13.6
+        """
+        if self.dispatcher.persistence:
+            if self.dispatcher.persistence.store_bot_data:
+                self.dispatcher.persistence.refresh_bot_data(self.bot_data)
+            if self.dispatcher.persistence.store_chat_data and self._chat_id_and_data is not None:
+                self.dispatcher.persistence.refresh_chat_data(*self._chat_id_and_data)
+            if self.dispatcher.persistence.store_user_data and self._user_id_and_data is not None:
+                self.dispatcher.persistence.refresh_user_data(*self._user_id_and_data)
+
+    def drop_callback_data(self, callback_query: CallbackQuery) -> None:
+        """
+        Deletes the cached data for the specified callback query.
+
+        .. versionadded:: 13.6
+
+        Note:
+            Will *not* raise exceptions in case the data is not found in the cache.
+            *Will* raise :class:`KeyError` in case the callback query can not be found in the
+            cache.
+
+        Args:
+            callback_query (:class:`telegram.CallbackQuery`): The callback query.
+
+        Raises:
+            KeyError | RuntimeError: :class:`KeyError`, if the callback query can not be found in
+                the cache and :class:`RuntimeError`, if the bot doesn't allow for arbitrary
+                callback data.
+        """
+        if isinstance(self.bot, ExtBot):
+            if not self.bot.arbitrary_callback_data:
+                raise RuntimeError(
+                    'This telegram.ext.ExtBot instance does not use arbitrary callback data.'
+                )
+            self.bot.callback_data_cache.drop_data(callback_query)
+        else:
+            raise RuntimeError('telegram.Bot does not allow for arbitrary callback data.')
+
     @classmethod
     def from_error(
-        cls,
+        cls: Type[CC],
         update: object,
         error: Exception,
         dispatcher: 'Dispatcher',
         async_args: Union[List, Tuple] = None,
         async_kwargs: Dict[str, object] = None,
-    ) -> 'CallbackContext':
+    ) -> CC:
+        """
+        Constructs an instance of :class:`telegram.ext.CallbackContext` to be passed to the error
+        handlers.
+
+        .. seealso:: :meth:`telegram.ext.Dispatcher.add_error_handler`
+
+        Args:
+            update (:obj:`object` | :class:`telegram.Update`): The update associated with the
+                error. May be :obj:`None`, e.g. for errors in job callbacks.
+            error (:obj:`Exception`): The error.
+            dispatcher (:class:`telegram.ext.Dispatcher`): The dispatcher associated with this
+                context.
+            async_args (List[:obj:`object`]): Optional. Positional arguments of the function that
+                raised the error. Pass only when the raising function was run asynchronously using
+                :meth:`telegram.ext.Dispatcher.run_async`.
+            async_kwargs (Dict[:obj:`str`, :obj:`object`]): Optional. Keyword arguments of the
+                function that raised the error. Pass only when the raising function was run
+                asynchronously using :meth:`telegram.ext.Dispatcher.run_async`.
+
+        Returns:
+            :class:`telegram.ext.CallbackContext`
+        """
         self = cls.from_update(update, dispatcher)
         self.error = error
         self.async_args = async_args
@@ -157,7 +261,21 @@ class CallbackContext:
         return self
 
     @classmethod
-    def from_update(cls, update: object, dispatcher: 'Dispatcher') -> 'CallbackContext':
+    def from_update(cls: Type[CC], update: object, dispatcher: 'Dispatcher') -> CC:
+        """
+        Constructs an instance of :class:`telegram.ext.CallbackContext` to be passed to the
+        handlers.
+
+        .. seealso:: :meth:`telegram.ext.Dispatcher.add_handler`
+
+        Args:
+            update (:obj:`object` | :class:`telegram.Update`): The update.
+            dispatcher (:class:`telegram.ext.Dispatcher`): The dispatcher associated with this
+                context.
+
+        Returns:
+            :class:`telegram.ext.CallbackContext`
+        """
         self = cls(dispatcher)
 
         if update is not None and isinstance(update, Update):
@@ -165,19 +283,45 @@ class CallbackContext:
             user = update.effective_user
 
             if chat:
-                self._chat_data = dispatcher.chat_data[chat.id]  # pylint: disable=W0212
+                self._chat_id_and_data = (
+                    chat.id,
+                    dispatcher.chat_data[chat.id],  # pylint: disable=W0212
+                )
             if user:
-                self._user_data = dispatcher.user_data[user.id]  # pylint: disable=W0212
+                self._user_id_and_data = (
+                    user.id,
+                    dispatcher.user_data[user.id],  # pylint: disable=W0212
+                )
         return self
 
     @classmethod
-    def from_job(cls, job: 'Job', dispatcher: 'Dispatcher') -> 'CallbackContext':
+    def from_job(cls: Type[CC], job: 'Job', dispatcher: 'Dispatcher') -> CC:
+        """
+        Constructs an instance of :class:`telegram.ext.CallbackContext` to be passed to a
+        job callback.
+
+        .. seealso:: :meth:`telegram.ext.JobQueue`
+
+        Args:
+            job (:class:`telegram.ext.Job`): The job.
+            dispatcher (:class:`telegram.ext.Dispatcher`): The dispatcher associated with this
+                context.
+
+        Returns:
+            :class:`telegram.ext.CallbackContext`
+        """
         self = cls(dispatcher)
         self.job = job
         return self
 
     def update(self, data: Dict[str, object]) -> None:
-        self.__dict__.update(data)
+        """Updates ``self.__slots__`` with the passed data.
+
+        Args:
+            data (Dict[:obj:`str`, :obj:`object`]): The data.
+        """
+        for key, value in data.items():
+            setattr(self, key, value)
 
     @property
     def bot(self) -> 'Bot':
