@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -356,7 +356,7 @@ class Dispatcher(Generic[CCT, UD, CD, BD]):
         raise RuntimeError(f'{cls.__name__} not initialized or multiple instances exist')
 
     def _pooled(self) -> None:
-        thr_name = current_thread().getName()
+        thr_name = current_thread().name
         while 1:
             promise = self.__async_queue.get()
 
