@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -17,22 +17,25 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains the base class for handlers as used by the Dispatcher."""
-
 from abc import ABC, abstractmethod
 from typing import TYPE_CHECKING, Any, Callable, Dict, Optional, TypeVar, Union, Generic
+from sys import version_info as py_ver
+
+from telegram.utils.deprecate import set_new_attribute_deprecated
 
 from telegram import Update
 from telegram.ext.utils.promise import Promise
 from telegram.utils.helpers import DefaultValue, DEFAULT_FALSE
+from telegram.ext.utils.types import CCT
 
 if TYPE_CHECKING:
-    from telegram.ext import CallbackContext, Dispatcher
+    from telegram.ext import Dispatcher
 
 RT = TypeVar('RT')
 UT = TypeVar('UT')
 
 
-class Handler(Generic[UT], ABC):
+class Handler(Generic[UT, CCT], ABC):
     """The base class for all update handlers. Create custom handlers by inheriting from it.
 
     Note:
@@ -90,9 +93,30 @@ class Handler(Generic[UT], ABC):
 
     """
 
+    # Apparently Py 3.7 and below have '__dict__' in ABC
+    if py_ver < (3, 7):
+        __slots__ = (
+            'callback',
+            'pass_update_queue',
+            'pass_job_queue',
+            'pass_user_data',
+            'pass_chat_data',
+            'run_async',
+        )
+    else:
+        __slots__ = (
+            'callback',  # type: ignore[assignment]
+            'pass_update_queue',
+            'pass_job_queue',
+            'pass_user_data',
+            'pass_chat_data',
+            'run_async',
+            '__dict__',
+        )
+
     def __init__(
         self,
-        callback: Callable[[UT, 'CallbackContext'], RT],
+        callback: Callable[[UT, CCT], RT],
         pass_update_queue: bool = False,
         pass_job_queue: bool = False,
         pass_user_data: bool = False,
@@ -106,6 +130,17 @@ class Handler(Generic[UT], ABC):
         self.pass_chat_data = pass_chat_data
         self.run_async = run_async
 
+    def __setattr__(self, key: str, value: object) -> None:
+        # See comment on BaseFilter to know why this was done.
+        if key.startswith('__'):
+            key = f"_{self.__class__.__name__}{key}"
+        if issubclass(self.__class__, Handler) and not self.__class__.__module__.startswith(
+            'telegram.ext.'
+        ):
+            object.__setattr__(self, key, value)
+            return
+        set_new_attribute_deprecated(self, key, value)
+
     @abstractmethod
     def check_update(self, update: object) -> Optional[Union[bool, object]]:
         """
@@ -131,7 +166,7 @@ class Handler(Generic[UT], ABC):
         update: UT,
         dispatcher: 'Dispatcher',
         check_result: object,
-        context: 'CallbackContext' = None,
+        context: CCT = None,
     ) -> Union[RT, Promise]:
         """
         This method is called if it was determined that an update should indeed
@@ -149,9 +184,12 @@ class Handler(Generic[UT], ABC):
 
         """
         run_async = self.run_async
-        if self.run_async is DEFAULT_FALSE and dispatcher.bot.defaults:
-            if dispatcher.bot.defaults.run_async:
-                run_async = True
+        if (
+            self.run_async is DEFAULT_FALSE
+            and dispatcher.bot.defaults
+            and dispatcher.bot.defaults.run_async
+        ):
+            run_async = True
 
         if context:
             self.collect_additional_context(context, update, dispatcher, check_result)
@@ -168,7 +206,7 @@ class Handler(Generic[UT], ABC):
 
     def collect_additional_context(
         self,
-        context: 'CallbackContext',
+        context: CCT,
         update: UT,
         dispatcher: 'Dispatcher',
         check_result: Any,
