@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,15 +23,19 @@ from typing import TYPE_CHECKING, Callable, Dict, List, Optional, TypeVar, Union
 from telegram.utils.helpers import DefaultValue, DEFAULT_FALSE
 
 from .handler import Handler
+from .utils.types import CCT
 
 if TYPE_CHECKING:
-    from telegram.ext import CallbackContext, Dispatcher
+    from telegram.ext import Dispatcher
 
 RT = TypeVar('RT')
 
 
-class StringCommandHandler(Handler[str]):
+class StringCommandHandler(Handler[str, CCT]):
     """Handler class to handle string commands. Commands are string updates that start with ``/``.
+    The handler will add a ``list`` to the
+    :class:`CallbackContext` named :attr:`CallbackContext.args`. It will contain a list of strings,
+    which is the text following the command split on single whitespace characters.
 
     Note:
         This handler is not used to handle Telegram :attr:`telegram.Update`, but strings manually
@@ -82,10 +86,12 @@ class StringCommandHandler(Handler[str]):
 
     """
 
+    __slots__ = ('command', 'pass_args')
+
     def __init__(
         self,
         command: str,
-        callback: Callable[[str, 'CallbackContext'], RT],
+        callback: Callable[[str, CCT], RT],
         pass_args: bool = False,
         pass_update_queue: bool = False,
         pass_job_queue: bool = False,
@@ -122,6 +128,9 @@ class StringCommandHandler(Handler[str]):
         update: str = None,
         check_result: Optional[List[str]] = None,
     ) -> Dict[str, object]:
+        """Provide text after the command to the callback the ``args`` argument as list, split on
+        single whitespaces.
+        """
         optional_args = super().collect_optional_args(dispatcher, update, check_result)
         if self.pass_args:
             optional_args['args'] = check_result
@@ -129,9 +138,12 @@ class StringCommandHandler(Handler[str]):
 
     def collect_additional_context(
         self,
-        context: 'CallbackContext',
+        context: CCT,
         update: str,
         dispatcher: 'Dispatcher',
         check_result: Optional[List[str]],
     ) -> None:
+        """Add text after the command to :attr:`CallbackContext.args` as list, split on single
+        whitespaces.
+        """
         context.args = check_result
