@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -56,6 +56,16 @@ class Location(TelegramObject):
 
     """
 
+    __slots__ = (
+        'longitude',
+        'horizontal_accuracy',
+        'proximity_alert_radius',
+        'live_period',
+        'latitude',
+        'heading',
+        '_id_attrs',
+    )
+
     def __init__(
         self,
         longitude: float,
