@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -34,6 +34,11 @@ class Sticker(TelegramObject):
     Objects of this class are comparable in terms of equality. Two objects of this class are
     considered equal, if their :attr:`file_unique_id` is equal.
 
+    Note:
+        As of v13.11 ``is_video`` is a required argument and therefore the order of the
+        arguments had to be changed. Use keyword arguments to make sure that the arguments are
+        passed correctly.
+
     Args:
         file_id (:obj:`str`): Identifier for this file, which can be used to download
             or reuse the file.
@@ -43,6 +48,9 @@ class Sticker(TelegramObject):
         width (:obj:`int`): Sticker width.
         height (:obj:`int`): Sticker height.
         is_animated (:obj:`bool`): :obj:`True`, if the sticker is animated.
+        is_video (:obj:`bool`): :obj:`True`, if the sticker is a video sticker.
+
+            .. versionadded:: 13.11
         thumb (:class:`telegram.PhotoSize`, optional): Sticker thumbnail in the .WEBP or .JPG
             format.
         emoji (:obj:`str`, optional): Emoji associated with the sticker
@@ -62,6 +70,9 @@ class Sticker(TelegramObject):
         width (:obj:`int`): Sticker width.
         height (:obj:`int`): Sticker height.
         is_animated (:obj:`bool`): :obj:`True`, if the sticker is animated.
+        is_video (:obj:`bool`): :obj:`True`, if the sticker is a video sticker.
+
+            .. versionadded:: 13.11
         thumb (:class:`telegram.PhotoSize`): Optional. Sticker thumbnail in the .webp or .jpg
             format.
         emoji (:obj:`str`): Optional. Emoji associated with the sticker.
@@ -73,6 +84,22 @@ class Sticker(TelegramObject):
 
     """
 
+    __slots__ = (
+        'bot',
+        'width',
+        'file_id',
+        'is_animated',
+        'is_video',
+        'file_size',
+        'thumb',
+        'set_name',
+        'mask_position',
+        'height',
+        'file_unique_id',
+        'emoji',
+        '_id_attrs',
+    )
+
     def __init__(
         self,
         file_id: str,
@@ -80,6 +107,7 @@ class Sticker(TelegramObject):
         width: int,
         height: int,
         is_animated: bool,
+        is_video: bool,
         thumb: PhotoSize = None,
         emoji: str = None,
         file_size: int = None,
@@ -94,6 +122,7 @@ class Sticker(TelegramObject):
         self.width = int(width)
         self.height = int(height)
         self.is_animated = is_animated
+        self.is_video = is_video
         # Optionals
         self.thumb = thumb
         self.emoji = emoji
@@ -106,7 +135,8 @@ class Sticker(TelegramObject):
 
     @classmethod
     def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['Sticker']:
-        data = cls.parse_data(data)
+        """See :meth:`telegram.TelegramObject.de_json`."""
+        data = cls._parse_data(data)
 
         if not data:
             return None
@@ -139,26 +169,48 @@ class StickerSet(TelegramObject):
     Objects of this class are comparable in terms of equality. Two objects of this class are
     considered equal, if their :attr:`name` is equal.
 
-    Attributes:
-        name (:obj:`str`): Sticker set name.
-        title (:obj:`str`): Sticker set title.
-        is_animated (:obj:`bool`): :obj:`True`, if the sticker set contains animated stickers.
-        contains_masks (:obj:`bool`): :obj:`True`, if the sticker set contains masks.
-        stickers (List[:class:`telegram.Sticker`]): List of all set stickers.
-        thumb (:class:`telegram.PhotoSize`): Optional. Sticker set thumbnail in the .WEBP or .TGS
-            format.
+    Note:
+        As of v13.11 ``is_video`` is a required argument and therefore the order of the
+        arguments had to be changed. Use keyword arguments to make sure that the arguments are
+        passed correctly.
 
     Args:
         name (:obj:`str`): Sticker set name.
         title (:obj:`str`): Sticker set title.
         is_animated (:obj:`bool`): :obj:`True`, if the sticker set contains animated stickers.
+        is_video (:obj:`bool`): :obj:`True`, if the sticker set contains video stickers.
+
+            .. versionadded:: 13.11
         contains_masks (:obj:`bool`): :obj:`True`, if the sticker set contains masks.
         stickers (List[:class:`telegram.Sticker`]): List of all set stickers.
-        thumb (:class:`telegram.PhotoSize`, optional): Sticker set thumbnail in the .WEBP or .TGS
-            format.
+        thumb (:class:`telegram.PhotoSize`, optional): Sticker set thumbnail in the ``.WEBP``,
+            ``.TGS``, or ``.WEBM`` format.
+
+    Attributes:
+        name (:obj:`str`): Sticker set name.
+        title (:obj:`str`): Sticker set title.
+        is_animated (:obj:`bool`): :obj:`True`, if the sticker set contains animated stickers.
+        is_video (:obj:`bool`): :obj:`True`, if the sticker set contains video stickers.
+
+            .. versionadded:: 13.11
+        contains_masks (:obj:`bool`): :obj:`True`, if the sticker set contains masks.
+        stickers (List[:class:`telegram.Sticker`]): List of all set stickers.
+        thumb (:class:`telegram.PhotoSize`): Optional. Sticker set thumbnail in the ``.WEBP``,
+            ``.TGS`` or ``.WEBM`` format.
 
     """
 
+    __slots__ = (
+        'is_animated',
+        'is_video',
+        'contains_masks',
+        'thumb',
+        'title',
+        'stickers',
+        'name',
+        '_id_attrs',
+    )
+
     def __init__(
         self,
         name: str,
@@ -166,12 +218,14 @@ class StickerSet(TelegramObject):
         is_animated: bool,
         contains_masks: bool,
         stickers: List[Sticker],
+        is_video: bool,
         thumb: PhotoSize = None,
         **_kwargs: Any,
     ):
         self.name = name
         self.title = title
         self.is_animated = is_animated
+        self.is_video = is_video
         self.contains_masks = contains_masks
         self.stickers = stickers
         # Optionals
@@ -181,6 +235,7 @@ class StickerSet(TelegramObject):
 
     @classmethod
     def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['StickerSet']:
+        """See :meth:`telegram.TelegramObject.de_json`."""
         if not data:
             return None
 
@@ -190,6 +245,7 @@ class StickerSet(TelegramObject):
         return cls(bot=bot, **data)
 
     def to_dict(self) -> JSONDict:
+        """See :meth:`telegram.TelegramObject.to_dict`."""
         data = super().to_dict()
 
         data['stickers'] = [s.to_dict() for s in data.get('stickers')]
@@ -230,6 +286,8 @@ class MaskPosition(TelegramObject):
 
     """
 
+    __slots__ = ('point', 'scale', 'x_shift', 'y_shift', '_id_attrs')
+
     FOREHEAD: ClassVar[str] = constants.STICKER_FOREHEAD
     """:const:`telegram.constants.STICKER_FOREHEAD`"""
     EYES: ClassVar[str] = constants.STICKER_EYES
@@ -249,7 +307,8 @@ class MaskPosition(TelegramObject):
 
     @classmethod
     def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['MaskPosition']:
-        data = cls.parse_data(data)
+        """See :meth:`telegram.TelegramObject.de_json`."""
+        data = cls._parse_data(data)
 
         if data is None:
             return None
