@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -35,16 +35,40 @@ class InlineKeyboardButton(TelegramObject):
     and :attr:`pay` are equal.
 
     Note:
-        You must use exactly one of the optional fields. Mind that :attr:`callback_game` is not
-        working as expected. Putting a game short name in it might, but is not guaranteed to work.
+        * You must use exactly one of the optional fields. Mind that :attr:`callback_game` is not
+          working as expected. Putting a game short name in it might, but is not guaranteed to
+          work.
+        * If your bot allows for arbitrary callback data, in keyboards returned in a response
+          from telegram, :attr:`callback_data` maybe be an instance of
+          :class:`telegram.ext.InvalidCallbackData`. This will be the case, if the data
+          associated with the button was already deleted.
+
+          .. versionadded:: 13.6
+
+        * Since Bot API 5.5, it's now allowed to mention users by their ID in inline keyboards.
+          This will only work in Telegram versions released after December 7, 2021.
+          Older clients will display *unsupported message*.
+
+    Warning:
+        If your bot allows your arbitrary callback data, buttons whose callback data is a
+        non-hashable object will become unhashable. Trying to evaluate ``hash(button)`` will
+        result in a :class:`TypeError`.
+
+        .. versionchanged:: 13.6
 
     Args:
         text (:obj:`str`): Label text on the button.
-        url (:obj:`str`): HTTP or tg:// url to be opened when button is pressed.
+        url (:obj:`str`, optional): HTTP or tg:// url to be opened when the button is pressed.
+            Links ``tg://user?id=<user_id>`` can be used to mention a user by
+            their ID without using a username, if this is allowed by their privacy settings.
+
+            .. versionchanged:: 13.9
+               You can now mention a user using ``tg://user?id=<user_id>``.
         login_url (:class:`telegram.LoginUrl`, optional): An HTTP URL used to automatically
             authorize the user. Can be used as a replacement for the Telegram Login Widget.
-        callback_data (:obj:`str`, optional): Data to be sent in a callback query to the bot when
-            button is pressed, UTF-8 1-64 bytes.
+        callback_data (:obj:`str` | :obj:`Any`, optional): Data to be sent in a callback query to
+            the bot when button is pressed, UTF-8 1-64 bytes. If the bot instance allows arbitrary
+            callback data, anything can be passed.
         switch_inline_query (:obj:`str`, optional): If set, pressing the button will prompt the
             user to select one of their chats, open that chat and insert the bot's username and the
             specified inline query in the input field. Can be empty, in which case just the bot's
@@ -61,16 +85,22 @@ class InlineKeyboardButton(TelegramObject):
             be launched when the user presses the button. This type of button must always be
             the ``first`` button in the first row.
         pay (:obj:`bool`, optional): Specify :obj:`True`, to send a Pay button. This type of button
-            must always be the ``first`` button in the first row.
+            must always be the `first` button in the first row and can only be used in invoice
+            messages.
         **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
     Attributes:
         text (:obj:`str`): Label text on the button.
-        url (:obj:`str`): Optional. HTTP or tg:// url to be opened when button is pressed.
+        url (:obj:`str`): Optional. HTTP or tg:// url to be opened when the button is pressed.
+            Links ``tg://user?id=<user_id>`` can be used to mention a user by
+            their ID without using a username, if this is allowed by their privacy settings.
+
+            .. versionchanged:: 13.9
+               You can now mention a user using ``tg://user?id=<user_id>``.
         login_url (:class:`telegram.LoginUrl`): Optional. An HTTP URL used to automatically
             authorize the user. Can be used as a replacement for the Telegram Login Widget.
-        callback_data (:obj:`str`): Optional. Data to be sent in a callback query to the bot when
-            button is pressed, UTF-8 1-64 bytes.
+        callback_data (:obj:`str` | :obj:`object`): Optional. Data to be sent in a callback query
+            to the bot when button is pressed, UTF-8 1-64 bytes.
         switch_inline_query (:obj:`str`): Optional. Will prompt the user to select one of their
             chats, open that chat and insert the bot's username and the specified inline query in
             the input field. Can be empty, in which case just the bot’s username will be inserted.
@@ -83,11 +113,23 @@ class InlineKeyboardButton(TelegramObject):
 
     """
 
+    __slots__ = (
+        'callback_game',
+        'url',
+        'switch_inline_query_current_chat',
+        'callback_data',
+        'pay',
+        'switch_inline_query',
+        'text',
+        '_id_attrs',
+        'login_url',
+    )
+
     def __init__(
         self,
         text: str,
         url: str = None,
-        callback_data: str = None,
+        callback_data: object = None,
         switch_inline_query: str = None,
         switch_inline_query_current_chat: str = None,
         callback_game: 'CallbackGame' = None,
@@ -106,7 +148,10 @@ class InlineKeyboardButton(TelegramObject):
         self.switch_inline_query_current_chat = switch_inline_query_current_chat
         self.callback_game = callback_game
         self.pay = pay
+        self._id_attrs = ()
+        self._set_id_attrs()
 
+    def _set_id_attrs(self) -> None:
         self._id_attrs = (
             self.text,
             self.url,
@@ -117,3 +162,16 @@ class InlineKeyboardButton(TelegramObject):
             self.callback_game,
             self.pay,
         )
+
+    def update_callback_data(self, callback_data: object) -> None:
+        """
+        Sets :attr:`callback_data` to the passed object. Intended to be used by
+        :class:`telegram.ext.CallbackDataCache`.
+
+        .. versionadded:: 13.6
+
+        Args:
+            callback_data (:obj:`obj`): The new callback data.
+        """
+        self.callback_data = callback_data
+        self._set_id_attrs()
