@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -25,7 +25,10 @@ class InputMessageContent(TelegramObject):
     """Base class for Telegram InputMessageContent Objects.
 
     See: :class:`telegram.InputContactMessageContent`,
+    :class:`telegram.InputInvoiceMessageContent`,
     :class:`telegram.InputLocationMessageContent`, :class:`telegram.InputTextMessageContent` and
     :class:`telegram.InputVenueMessageContent` for more details.
 
     """
+
+    __slots__ = ()
