@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -17,9 +17,10 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains an object that represents a Telegram ReplyKeyboardRemove."""
-from telegram import ReplyMarkup
 from typing import Any
 
+from telegram import ReplyMarkup
+
 
 class ReplyKeyboardRemove(ReplyMarkup):
     """
@@ -28,16 +29,15 @@ class ReplyKeyboardRemove(ReplyMarkup):
     until a new keyboard is sent by a bot. An exception is made for one-time keyboards that are
     hidden immediately after the user presses a button (see :class:`telegram.ReplyKeyboardMarkup`).
 
-    Attributes:
-        remove_keyboard (:obj:`True`): Requests clients to remove the custom keyboard.
-        selective (:obj:`bool`): Optional. Use this parameter if you want to remove the keyboard
-            for specific users only.
-
     Example:
         A user votes in a poll, bot returns confirmation message in reply to the vote and removes
         the keyboard for that user, while still showing the keyboard with poll options to users who
         haven't voted yet.
 
+    Note:
+        User will not be able to summon this keyboard; if you want to hide the keyboard from
+        sight but keep it accessible, use :attr:`telegram.ReplyKeyboardMarkup.one_time_keyboard`.
+
     Args:
         selective (:obj:`bool`, optional): Use this parameter if you want to remove the keyboard
             for specific users only. Targets:
@@ -48,13 +48,16 @@ class ReplyKeyboardRemove(ReplyMarkup):
 
         **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
-    Note:
-        User will not be able to summon this keyboard; if you want to hide the keyboard from
-        sight but keep it accessible, use :attr:`telegram.ReplyKeyboardMarkup.one_time_keyboard`.
+    Attributes:
+        remove_keyboard (:obj:`True`): Requests clients to remove the custom keyboard.
+        selective (:obj:`bool`): Optional. Use this parameter if you want to remove the keyboard
+            for specific users only.
 
     """
 
-    def __init__(self, selective: bool = False, **kwargs: Any):
+    __slots__ = ('selective', 'remove_keyboard')
+
+    def __init__(self, selective: bool = False, **_kwargs: Any):
         # Required
         self.remove_keyboard = True
         # Optionals
