@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -72,6 +72,15 @@ def callback_query(bot):
 class TestCallbackQueryHandler:
     test_flag = False
 
+    def test_slot_behaviour(self, recwarn, mro_slots):
+        handler = CallbackQueryHandler(self.callback_data_1, pass_user_data=True)
+        for attr in handler.__slots__:
+            assert getattr(handler, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not handler.__dict__, f"got missing slot(s): {handler.__dict__}"
+        assert len(mro_slots(handler)) == len(set(mro_slots(handler))), "duplicate slot"
+        handler.custom, handler.callback = 'should give warning', self.callback_basic
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     @pytest.fixture(autouse=True)
     def reset(self):
         self.test_flag = False
@@ -135,6 +144,42 @@ class TestCallbackQueryHandler:
         callback_query.callback_query.data = 'nothing here'
         assert not handler.check_update(callback_query)
 
+        callback_query.callback_query.data = None
+        callback_query.callback_query.game_short_name = "this is a short game name"
+        assert not handler.check_update(callback_query)
+
+    def test_with_callable_pattern(self, callback_query):
+        class CallbackData:
+            pass
+
+        def pattern(callback_data):
+            return isinstance(callback_data, CallbackData)
+
+        handler = CallbackQueryHandler(self.callback_basic, pattern=pattern)
+
+        callback_query.callback_query.data = CallbackData()
+        assert handler.check_update(callback_query)
+        callback_query.callback_query.data = 'callback_data'
+        assert not handler.check_update(callback_query)
+
+    def test_with_type_pattern(self, callback_query):
+        class CallbackData:
+            pass
+
+        handler = CallbackQueryHandler(self.callback_basic, pattern=CallbackData)
+
+        callback_query.callback_query.data = CallbackData()
+        assert handler.check_update(callback_query)
+        callback_query.callback_query.data = 'callback_data'
+        assert not handler.check_update(callback_query)
+
+        handler = CallbackQueryHandler(self.callback_basic, pattern=bool)
+
+        callback_query.callback_query.data = False
+        assert handler.check_update(callback_query)
+        callback_query.callback_query.data = 'callback_data'
+        assert not handler.check_update(callback_query)
+
     def test_with_passing_group_dict(self, dp, callback_query):
         handler = CallbackQueryHandler(
             self.callback_group, pattern='(?P<begin>.*)est(?P<end>.*)', pass_groups=True
@@ -230,3 +275,18 @@ class TestCallbackQueryHandler:
 
         cdp.process_update(callback_query)
         assert self.test_flag
+
+    def test_context_callable_pattern(self, cdp, callback_query):
+        class CallbackData:
+            pass
+
+        def pattern(callback_data):
+            return isinstance(callback_data, CallbackData)
+
+        def callback(update, context):
+            assert context.matches is None
+
+        handler = CallbackQueryHandler(callback, pattern=pattern)
+        cdp.add_handler(handler)
+
+        cdp.process_update(callback_query)
