@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -37,9 +37,12 @@ def chat(bot):
         can_set_sticker_set=TestChat.can_set_sticker_set,
         permissions=TestChat.permissions,
         slow_mode_delay=TestChat.slow_mode_delay,
+        message_auto_delete_time=TestChat.message_auto_delete_time,
         bio=TestChat.bio,
         linked_chat_id=TestChat.linked_chat_id,
         location=TestChat.location,
+        has_private_forwards=True,
+        has_protected_content=True,
     )
 
 
@@ -57,9 +60,20 @@ class TestChat:
         can_invite_users=True,
     )
     slow_mode_delay = 30
+    message_auto_delete_time = 42
     bio = "I'm a Barbie Girl in a Barbie World"
     linked_chat_id = 11880
     location = ChatLocation(Location(123, 456), 'Barbie World')
+    has_protected_content = True
+    has_private_forwards = True
+
+    def test_slot_behaviour(self, chat, recwarn, mro_slots):
+        for attr in chat.__slots__:
+            assert getattr(chat, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not chat.__dict__, f"got missing slot(s): {chat.__dict__}"
+        assert len(mro_slots(chat)) == len(set(mro_slots(chat))), "duplicate slot"
+        chat.custom, chat.id = 'should give warning', self.id_
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
 
     def test_de_json(self, bot):
         json_dict = {
@@ -72,7 +86,10 @@ class TestChat:
             'can_set_sticker_set': self.can_set_sticker_set,
             'permissions': self.permissions.to_dict(),
             'slow_mode_delay': self.slow_mode_delay,
+            'message_auto_delete_time': self.message_auto_delete_time,
             'bio': self.bio,
+            'has_protected_content': self.has_protected_content,
+            'has_private_forwards': self.has_private_forwards,
             'linked_chat_id': self.linked_chat_id,
             'location': self.location.to_dict(),
         }
@@ -87,7 +104,10 @@ class TestChat:
         assert chat.can_set_sticker_set == self.can_set_sticker_set
         assert chat.permissions == self.permissions
         assert chat.slow_mode_delay == self.slow_mode_delay
+        assert chat.message_auto_delete_time == self.message_auto_delete_time
         assert chat.bio == self.bio
+        assert chat.has_protected_content == self.has_protected_content
+        assert chat.has_private_forwards == self.has_private_forwards
         assert chat.linked_chat_id == self.linked_chat_id
         assert chat.location.location == self.location.location
         assert chat.location.address == self.location.address
@@ -103,7 +123,10 @@ class TestChat:
         assert chat_dict['all_members_are_administrators'] == chat.all_members_are_administrators
         assert chat_dict['permissions'] == chat.permissions.to_dict()
         assert chat_dict['slow_mode_delay'] == chat.slow_mode_delay
+        assert chat_dict['message_auto_delete_time'] == chat.message_auto_delete_time
         assert chat_dict['bio'] == chat.bio
+        assert chat_dict['has_private_forwards'] == chat.has_private_forwards
+        assert chat_dict['has_protected_content'] == chat.has_protected_content
         assert chat_dict['linked_chat_id'] == chat.linked_chat_id
         assert chat_dict['location'] == chat.location.to_dict()
 
@@ -163,18 +186,27 @@ class TestChat:
         monkeypatch.setattr(chat.bot, 'get_chat_administrators', make_assertion)
         assert chat.get_administrators()
 
-    def test_get_members_count(self, monkeypatch, chat):
+    def test_get_member_count(self, monkeypatch, chat):
         def make_assertion(*_, **kwargs):
             return kwargs['chat_id'] == chat.id
 
         assert check_shortcut_signature(
-            Chat.get_members_count, Bot.get_chat_members_count, ['chat_id'], []
+            Chat.get_member_count, Bot.get_chat_member_count, ['chat_id'], []
         )
-        assert check_shortcut_call(chat.get_members_count, chat.bot, 'get_chat_members_count')
-        assert check_defaults_handling(chat.get_members_count, chat.bot)
+        assert check_shortcut_call(chat.get_member_count, chat.bot, 'get_chat_member_count')
+        assert check_defaults_handling(chat.get_member_count, chat.bot)
 
-        monkeypatch.setattr(chat.bot, 'get_chat_members_count', make_assertion)
+        monkeypatch.setattr(chat.bot, 'get_chat_member_count', make_assertion)
+        assert chat.get_member_count()
+
+    def test_get_members_count_warning(self, chat, monkeypatch, recwarn):
+        def make_assertion(*_, **kwargs):
+            return kwargs['chat_id'] == chat.id
+
+        monkeypatch.setattr(chat.bot, 'get_chat_member_count', make_assertion)
         assert chat.get_members_count()
+        assert len(recwarn) == 1
+        assert '`Chat.get_members_count` is deprecated' in str(recwarn[0].message)
 
     def test_get_member(self, monkeypatch, chat):
         def make_assertion(*_, **kwargs):
@@ -189,26 +221,68 @@ class TestChat:
         monkeypatch.setattr(chat.bot, 'get_chat_member', make_assertion)
         assert chat.get_member(user_id=42)
 
-    def test_kick_member(self, monkeypatch, chat):
+    def test_ban_member(self, monkeypatch, chat):
         def make_assertion(*_, **kwargs):
             chat_id = kwargs['chat_id'] == chat.id
             user_id = kwargs['user_id'] == 42
             until = kwargs['until_date'] == 43
             return chat_id and user_id and until
 
-        assert check_shortcut_signature(Chat.kick_member, Bot.kick_chat_member, ['chat_id'], [])
-        assert check_shortcut_call(chat.kick_member, chat.bot, 'kick_chat_member')
-        assert check_defaults_handling(chat.kick_member, chat.bot)
+        assert check_shortcut_signature(Chat.ban_member, Bot.ban_chat_member, ['chat_id'], [])
+        assert check_shortcut_call(chat.ban_member, chat.bot, 'ban_chat_member')
+        assert check_defaults_handling(chat.ban_member, chat.bot)
 
-        monkeypatch.setattr(chat.bot, 'kick_chat_member', make_assertion)
+        monkeypatch.setattr(chat.bot, 'ban_chat_member', make_assertion)
+        assert chat.ban_member(user_id=42, until_date=43)
+
+    def test_ban_sender_chat(self, monkeypatch, chat):
+        def make_assertion(*_, **kwargs):
+            chat_id = kwargs['chat_id'] == chat.id
+            sender_chat_id = kwargs['sender_chat_id'] == 42
+            return chat_id and sender_chat_id
+
+        assert check_shortcut_signature(
+            Chat.ban_sender_chat, Bot.ban_chat_sender_chat, ['chat_id'], []
+        )
+        assert check_shortcut_call(chat.ban_sender_chat, chat.bot, 'ban_chat_sender_chat')
+        assert check_defaults_handling(chat.ban_sender_chat, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'ban_chat_sender_chat', make_assertion)
+        assert chat.ban_sender_chat(42)
+
+    def test_ban_chat(self, monkeypatch, chat):
+        def make_assertion(*_, **kwargs):
+            chat_id = kwargs['chat_id'] == 42
+            sender_chat_id = kwargs['sender_chat_id'] == chat.id
+            return chat_id and sender_chat_id
+
+        assert check_shortcut_signature(
+            Chat.ban_chat, Bot.ban_chat_sender_chat, ['sender_chat_id'], []
+        )
+        assert check_shortcut_call(chat.ban_chat, chat.bot, 'ban_chat_sender_chat')
+        assert check_defaults_handling(chat.ban_chat, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'ban_chat_sender_chat', make_assertion)
+        assert chat.ban_chat(42)
+
+    def test_kick_member_warning(self, chat, monkeypatch, recwarn):
+        def make_assertion(*_, **kwargs):
+            chat_id = kwargs['chat_id'] == chat.id
+            user_id = kwargs['user_id'] == 42
+            until = kwargs['until_date'] == 43
+            return chat_id and user_id and until
+
+        monkeypatch.setattr(chat.bot, 'ban_chat_member', make_assertion)
         assert chat.kick_member(user_id=42, until_date=43)
+        assert len(recwarn) == 1
+        assert '`Chat.kick_member` is deprecated' in str(recwarn[0].message)
 
     @pytest.mark.parametrize('only_if_banned', [True, False, None])
     def test_unban_member(self, monkeypatch, chat, only_if_banned):
         def make_assertion(*_, **kwargs):
             chat_id = kwargs['chat_id'] == chat.id
             user_id = kwargs['user_id'] == 42
-            o_i_b = kwargs.get('only_if_banned', None) == only_if_banned
+            o_i_b = kwargs.get('only_if_banned') == only_if_banned
             return chat_id and user_id and o_i_b
 
         assert check_shortcut_signature(Chat.unban_member, Bot.unban_chat_member, ['chat_id'], [])
@@ -218,12 +292,42 @@ class TestChat:
         monkeypatch.setattr(chat.bot, 'unban_chat_member', make_assertion)
         assert chat.unban_member(user_id=42, only_if_banned=only_if_banned)
 
+    def test_unban_sender_chat(self, monkeypatch, chat):
+        def make_assertion(*_, **kwargs):
+            chat_id = kwargs['chat_id'] == chat.id
+            sender_chat_id = kwargs['sender_chat_id'] == 42
+            return chat_id and sender_chat_id
+
+        assert check_shortcut_signature(
+            Chat.unban_sender_chat, Bot.unban_chat_sender_chat, ['chat_id'], []
+        )
+        assert check_shortcut_call(chat.unban_sender_chat, chat.bot, 'unban_chat_sender_chat')
+        assert check_defaults_handling(chat.unban_sender_chat, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'unban_chat_sender_chat', make_assertion)
+        assert chat.unban_sender_chat(42)
+
+    def test_unban_chat(self, monkeypatch, chat):
+        def make_assertion(*_, **kwargs):
+            chat_id = kwargs['chat_id'] == 42
+            sender_chat_id = kwargs['sender_chat_id'] == chat.id
+            return chat_id and sender_chat_id
+
+        assert check_shortcut_signature(
+            Chat.unban_chat, Bot.ban_chat_sender_chat, ['sender_chat_id'], []
+        )
+        assert check_shortcut_call(chat.unban_chat, chat.bot, 'unban_chat_sender_chat')
+        assert check_defaults_handling(chat.unban_chat, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'unban_chat_sender_chat', make_assertion)
+        assert chat.unban_chat(42)
+
     @pytest.mark.parametrize('is_anonymous', [True, False, None])
     def test_promote_member(self, monkeypatch, chat, is_anonymous):
         def make_assertion(*_, **kwargs):
             chat_id = kwargs['chat_id'] == chat.id
             user_id = kwargs['user_id'] == 42
-            o_i_b = kwargs.get('is_anonymous', None) == is_anonymous
+            o_i_b = kwargs.get('is_anonymous') == is_anonymous
             return chat_id and user_id and o_i_b
 
         assert check_shortcut_signature(
@@ -241,7 +345,7 @@ class TestChat:
         def make_assertion(*_, **kwargs):
             chat_id = kwargs['chat_id'] == chat.id
             user_id = kwargs['user_id'] == 42
-            o_i_b = kwargs.get('permissions', None) == permissions
+            o_i_b = kwargs.get('permissions') == permissions
             return chat_id and user_id and o_i_b
 
         assert check_shortcut_signature(
@@ -411,18 +515,9 @@ class TestChat:
             description = kwargs['description'] == 'description'
             payload = kwargs['payload'] == 'payload'
             provider_token = kwargs['provider_token'] == 'provider_token'
-            start_parameter = kwargs['start_parameter'] == 'start_parameter'
             currency = kwargs['currency'] == 'currency'
             prices = kwargs['prices'] == 'prices'
-            args = (
-                title
-                and description
-                and payload
-                and provider_token
-                and start_parameter
-                and currency
-                and prices
-            )
+            args = title and description and payload and provider_token and currency and prices
             return kwargs['chat_id'] == chat.id and args
 
         assert check_shortcut_signature(Chat.send_invoice, Bot.send_invoice, ['chat_id'], [])
@@ -435,7 +530,6 @@ class TestChat:
             'description',
             'payload',
             'provider_token',
-            'start_parameter',
             'currency',
             'prices',
         )
@@ -556,6 +650,92 @@ class TestChat:
         monkeypatch.setattr(chat.bot, 'copy_message', make_assertion)
         assert chat.copy_message(chat_id='test_copy', message_id=42)
 
+    def test_export_invite_link(self, monkeypatch, chat):
+        def make_assertion(*_, **kwargs):
+            return kwargs['chat_id'] == chat.id
+
+        assert check_shortcut_signature(
+            Chat.export_invite_link, Bot.export_chat_invite_link, ['chat_id'], []
+        )
+        assert check_shortcut_call(chat.export_invite_link, chat.bot, 'export_chat_invite_link')
+        assert check_defaults_handling(chat.export_invite_link, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'export_chat_invite_link', make_assertion)
+        assert chat.export_invite_link()
+
+    def test_create_invite_link(self, monkeypatch, chat):
+        def make_assertion(*_, **kwargs):
+            return kwargs['chat_id'] == chat.id
+
+        assert check_shortcut_signature(
+            Chat.create_invite_link, Bot.create_chat_invite_link, ['chat_id'], []
+        )
+        assert check_shortcut_call(chat.create_invite_link, chat.bot, 'create_chat_invite_link')
+        assert check_defaults_handling(chat.create_invite_link, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'create_chat_invite_link', make_assertion)
+        assert chat.create_invite_link()
+
+    def test_edit_invite_link(self, monkeypatch, chat):
+        link = "ThisIsALink"
+
+        def make_assertion(*_, **kwargs):
+            return kwargs['chat_id'] == chat.id and kwargs['invite_link'] == link
+
+        assert check_shortcut_signature(
+            Chat.edit_invite_link, Bot.edit_chat_invite_link, ['chat_id'], []
+        )
+        assert check_shortcut_call(chat.edit_invite_link, chat.bot, 'edit_chat_invite_link')
+        assert check_defaults_handling(chat.edit_invite_link, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'edit_chat_invite_link', make_assertion)
+        assert chat.edit_invite_link(invite_link=link)
+
+    def test_revoke_invite_link(self, monkeypatch, chat):
+        link = "ThisIsALink"
+
+        def make_assertion(*_, **kwargs):
+            return kwargs['chat_id'] == chat.id and kwargs['invite_link'] == link
+
+        assert check_shortcut_signature(
+            Chat.revoke_invite_link, Bot.revoke_chat_invite_link, ['chat_id'], []
+        )
+        assert check_shortcut_call(chat.revoke_invite_link, chat.bot, 'revoke_chat_invite_link')
+        assert check_defaults_handling(chat.revoke_invite_link, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'revoke_chat_invite_link', make_assertion)
+        assert chat.revoke_invite_link(invite_link=link)
+
+    def test_approve_join_request(self, monkeypatch, chat):
+        def make_assertion(*_, **kwargs):
+            return kwargs['chat_id'] == chat.id and kwargs['user_id'] == 42
+
+        assert check_shortcut_signature(
+            Chat.approve_join_request, Bot.approve_chat_join_request, ['chat_id'], []
+        )
+        assert check_shortcut_call(
+            chat.approve_join_request, chat.bot, 'approve_chat_join_request'
+        )
+        assert check_defaults_handling(chat.approve_join_request, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'approve_chat_join_request', make_assertion)
+        assert chat.approve_join_request(user_id=42)
+
+    def test_decline_join_request(self, monkeypatch, chat):
+        def make_assertion(*_, **kwargs):
+            return kwargs['chat_id'] == chat.id and kwargs['user_id'] == 42
+
+        assert check_shortcut_signature(
+            Chat.decline_join_request, Bot.decline_chat_join_request, ['chat_id'], []
+        )
+        assert check_shortcut_call(
+            chat.decline_join_request, chat.bot, 'decline_chat_join_request'
+        )
+        assert check_defaults_handling(chat.decline_join_request, chat.bot)
+
+        monkeypatch.setattr(chat.bot, 'decline_chat_join_request', make_assertion)
+        assert chat.decline_join_request(user_id=42)
+
     def test_equality(self):
         a = Chat(self.id_, self.title, self.type_)
         b = Chat(self.id_, self.title, self.type_)
