@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,6 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
-
 import os
 import pytest
 from flaky import flaky
@@ -52,8 +51,15 @@ class TestChatPhoto:
     chatphoto_big_file_unique_id = 'bigadc3145fd2e84d95b64d68eaa22aa33e'
     chatphoto_file_url = 'https://python-telegram-bot.org/static/testfiles/telegram.jpg'
 
+    def test_slot_behaviour(self, chat_photo, recwarn, mro_slots):
+        for attr in chat_photo.__slots__:
+            assert getattr(chat_photo, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not chat_photo.__dict__, f"got missing slot(s): {chat_photo.__dict__}"
+        assert len(mro_slots(chat_photo)) == len(set(mro_slots(chat_photo))), "duplicate slot"
+        chat_photo.custom, chat_photo.big_file_id = 'gives warning', self.chatphoto_big_file_id
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_all_args(self, bot, super_group_id, chatphoto_file, chat_photo, thumb_file):
         def func():
             assert bot.set_chat_photo(super_group_id, chatphoto_file)
@@ -61,7 +67,6 @@ class TestChatPhoto:
         expect_bad_request(func, 'Type of file mismatch', 'Telegram did not accept the file.')
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_get_and_download(self, bot, chat_photo):
         new_file = bot.get_file(chat_photo.small_file_id)
 
@@ -112,7 +117,6 @@ class TestChatPhoto:
         assert chat_photo_dict['big_file_unique_id'] == chat_photo.big_file_unique_id
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_error_send_empty_file(self, bot, super_group_id):
         chatphoto_file = open(os.devnull, 'rb')
 
@@ -120,7 +124,6 @@ class TestChatPhoto:
             bot.set_chat_photo(chat_id=super_group_id, photo=chatphoto_file)
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_error_send_empty_file_id(self, bot, super_group_id):
         with pytest.raises(TelegramError):
             bot.set_chat_photo(chat_id=super_group_id, photo='')
