@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -79,7 +79,7 @@ class BaseTest:
 
     def make_callback_for(self, pass_keyword):
         def callback(bot, update, **kwargs):
-            self.test_flag = kwargs.get(keyword, None) is not None
+            self.test_flag = kwargs.get(keyword) is not None
 
         keyword = pass_keyword[5:]
         return callback
@@ -102,13 +102,13 @@ class BaseTest:
 
     def callback_context_regex1(self, update, context):
         if context.matches:
-            types = all([type(res) == self.SRE_TYPE for res in context.matches])
+            types = all(type(res) is self.SRE_TYPE for res in context.matches)
             num = len(context.matches) == 1
             self.test_flag = types and num
 
     def callback_context_regex2(self, update, context):
         if context.matches:
-            types = all([type(res) == self.SRE_TYPE for res in context.matches])
+            types = all(type(res) is self.SRE_TYPE for res in context.matches)
             num = len(context.matches) == 2
             self.test_flag = types and num
 
@@ -142,6 +142,15 @@ class BaseTest:
 class TestCommandHandler(BaseTest):
     CMD = '/test'
 
+    def test_slot_behaviour(self, recwarn, mro_slots):
+        handler = self.make_default_handler()
+        for attr in handler.__slots__:
+            assert getattr(handler, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not handler.__dict__, f"got missing slot(s): {handler.__dict__}"
+        assert len(mro_slots(handler)) == len(set(mro_slots(handler))), "duplicate slot"
+        handler.custom, handler.command = 'should give warning', self.CMD
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     @pytest.fixture(scope='class')
     def command(self):
         return self.CMD
@@ -296,6 +305,15 @@ class TestPrefixHandler(BaseTest):
     COMMANDS = ['help', 'test']
     COMBINATIONS = list(combinations(PREFIXES, COMMANDS))
 
+    def test_slot_behaviour(self, mro_slots, recwarn):
+        handler = self.make_default_handler()
+        for attr in handler.__slots__:
+            assert getattr(handler, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not handler.__dict__, f"got missing slot(s): {handler.__dict__}"
+        assert len(mro_slots(handler)) == len(set(mro_slots(handler))), "duplicate slot"
+        handler.custom, handler.command = 'should give warning', self.COMMANDS
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     @pytest.fixture(scope='class', params=PREFIXES)
     def prefix(self, request):
         return request.param
