@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -25,7 +25,6 @@ from telegram.error import BadRequest
 
 class TestConstants:
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_max_message_length(self, bot, chat_id):
         bot.send_message(chat_id=chat_id, text='a' * constants.MAX_MESSAGE_LENGTH)
 
@@ -36,7 +35,6 @@ class TestConstants:
             bot.send_message(chat_id=chat_id, text='a' * (constants.MAX_MESSAGE_LENGTH + 1))
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_max_caption_length(self, bot, chat_id):
         good_caption = 'a' * constants.MAX_CAPTION_LENGTH
         with open('tests/data/telegram.png', 'rb') as f:
@@ -47,6 +45,5 @@ class TestConstants:
         with pytest.raises(
             BadRequest,
             match="Media_caption_too_long",
-        ):
-            with open('tests/data/telegram.png', 'rb') as f:
-                bot.send_photo(photo=f, caption=bad_caption, chat_id=chat_id)
+        ), open('tests/data/telegram.png', 'rb') as f:
+            bot.send_photo(photo=f, caption=bad_caption, chat_id=chat_id)
