@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -53,6 +53,14 @@ class TestDocument:
     document_file_id = '5a3128a4d2a04750b5b58397f3b5e812'
     document_file_unique_id = 'adc3145fd2e84d95b64d68eaa22aa33e'
 
+    def test_slot_behaviour(self, document, recwarn, mro_slots):
+        for attr in document.__slots__:
+            assert getattr(document, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not document.__dict__, f"got missing slot(s): {document.__dict__}"
+        assert len(mro_slots(document)) == len(set(mro_slots(document))), "duplicate slot"
+        document.custom, document.file_name = 'should give warning', self.file_name
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), f"{recwarn}"
+
     def test_creation(self, document):
         assert isinstance(document, Document)
         assert isinstance(document.file_id, str)
@@ -69,13 +77,13 @@ class TestDocument:
         assert document.thumb.height == self.thumb_height
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_all_args(self, bot, chat_id, document_file, document, thumb_file):
         message = bot.send_document(
             chat_id,
             document=document_file,
             caption=self.caption,
             disable_notification=False,
+            protect_content=True,
             filename='telegram_custom.png',
             parse_mode='Markdown',
             thumb=thumb_file,
@@ -93,9 +101,9 @@ class TestDocument:
         assert message.caption == self.caption.replace('*', '')
         assert message.document.thumb.width == self.thumb_width
         assert message.document.thumb.height == self.thumb_height
+        assert message.has_protected_content
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_get_and_download(self, bot, document):
         new_file = bot.get_file(document.file_id)
 
@@ -109,7 +117,6 @@ class TestDocument:
         assert os.path.isfile('telegram.png')
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_url_gif_file(self, bot, chat_id):
         message = bot.send_document(chat_id, self.document_file_url)
 
@@ -126,7 +133,6 @@ class TestDocument:
         assert document.file_size == 3878
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_resend(self, bot, chat_id, document):
         message = bot.send_document(chat_id=chat_id, document=document.file_id)
 
@@ -153,7 +159,6 @@ class TestDocument:
         assert message
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_document_caption_entities(self, bot, chat_id, document):
         test_string = 'Italic Bold Code'
         entities = [
@@ -169,7 +174,6 @@ class TestDocument:
         assert message.caption_entities == entities
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize('default_bot', [{'parse_mode': 'Markdown'}], indirect=True)
     def test_send_document_default_parse_mode_1(self, default_bot, chat_id, document):
         test_string = 'Italic Bold Code'
@@ -180,7 +184,6 @@ class TestDocument:
         assert message.caption == test_string
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize('default_bot', [{'parse_mode': 'Markdown'}], indirect=True)
     def test_send_document_default_parse_mode_2(self, default_bot, chat_id, document):
         test_markdown_string = '_Italic_ *Bold* `Code`'
@@ -192,7 +195,6 @@ class TestDocument:
         assert message.caption_markdown == escape_markdown(test_markdown_string)
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize('default_bot', [{'parse_mode': 'Markdown'}], indirect=True)
     def test_send_document_default_parse_mode_3(self, default_bot, chat_id, document):
         test_markdown_string = '_Italic_ *Bold* `Code`'
@@ -204,7 +206,6 @@ class TestDocument:
         assert message.caption_markdown == escape_markdown(test_markdown_string)
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize(
         'default_bot,custom',
         [
@@ -251,6 +252,7 @@ class TestDocument:
         monkeypatch.setattr(bot, '_post', make_assertion)
         bot.send_document(chat_id, file, thumb=file)
         assert test_flag
+        monkeypatch.delattr(bot, '_post')
 
     def test_de_json(self, bot, document):
         json_dict = {
@@ -281,14 +283,11 @@ class TestDocument:
         assert document_dict['file_size'] == document.file_size
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_error_send_empty_file(self, bot, chat_id):
-        with open(os.devnull, 'rb') as f:
-            with pytest.raises(TelegramError):
-                bot.send_document(chat_id=chat_id, document=f)
+        with open(os.devnull, 'rb') as f, pytest.raises(TelegramError):
+            bot.send_document(chat_id=chat_id, document=f)
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_error_send_empty_file_id(self, bot, chat_id):
         with pytest.raises(TelegramError):
             bot.send_document(chat_id=chat_id, document='')
