@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,6 +22,8 @@ import pytest
 
 from telegram import Message, User, Chat, MessageEntity, Document, Update, Dice
 from telegram.ext import Filters, BaseFilter, MessageFilter, UpdateFilter
+from sys import version_info as py_ver
+import inspect
 import re
 
 from telegram.utils.deprecate import TelegramDeprecationWarning
@@ -38,13 +40,15 @@ def update():
             from_user=User(0, 'Testuser', False),
             via_bot=User(0, "Testbot", True),
             sender_chat=Chat(0, 'Channel'),
+            forward_from=User(0, "HAL9000", False),
+            forward_from_chat=Chat(0, "Channel"),
         ),
     )
 
 
 @pytest.fixture(scope='function', params=MessageEntity.ALL_TYPES)
 def message_entity(request):
-    return MessageEntity(request.param, 0, 0, url='', user='')
+    return MessageEntity(request.param, 0, 0, url='', user=User(1, 'first_name', False))
 
 
 @pytest.fixture(
@@ -57,6 +61,67 @@ def base_class(request):
 
 
 class TestFilters:
+    def test_all_filters_slot_behaviour(self, recwarn, mro_slots):
+        """
+        Use depth first search to get all nested filters, and instantiate them (which need it) with
+        the correct number of arguments, then test each filter separately. Also tests setting
+        custom attributes on custom filters.
+        """
+        # The total no. of filters excluding filters defined in __all__ is about 70 as of 16/2/21.
+        # Gather all the filters to test using DFS-
+        visited = []
+        classes = inspect.getmembers(Filters, predicate=inspect.isclass)  # List[Tuple[str, type]]
+        stack = classes.copy()
+        while stack:
+            cls = stack[-1][-1]  # get last element and its class
+            for inner_cls in inspect.getmembers(
+                cls,  # Get inner filters
+                lambda a: inspect.isclass(a) and not issubclass(a, cls.__class__),
+            ):
+                if inner_cls[1] not in visited:
+                    stack.append(inner_cls)
+                    visited.append(inner_cls[1])
+                    classes.append(inner_cls)
+                    break
+            else:
+                stack.pop()
+
+        # Now start the actual testing
+        for name, cls in classes:
+            # Can't instantiate abstract classes without overriding methods, so skip them for now
+            if inspect.isabstract(cls) or name in {'__class__', '__base__'}:
+                continue
+
+            assert '__slots__' in cls.__dict__, f"Filter {name!r} doesn't have __slots__"
+            # get no. of args minus the 'self' argument
+            args = len(inspect.signature(cls.__init__).parameters) - 1
+            if cls.__base__.__name__ == '_ChatUserBaseFilter':  # Special case, only 1 arg needed
+                inst = cls('1')
+            else:
+                inst = cls() if args < 1 else cls(*['blah'] * args)  # unpack variable no. of args
+
+            for attr in cls.__slots__:
+                assert getattr(inst, attr, 'err') != 'err', f"got extra slot '{attr}' for {name}"
+                assert not inst.__dict__, f"got missing slot(s): {inst.__dict__} for {name}"
+                assert len(mro_slots(inst)) == len(set(mro_slots(inst))), f"same slot in {name}"
+
+            with pytest.warns(TelegramDeprecationWarning, match='custom attributes') as warn:
+                inst.custom = 'should give warning'
+                if not warn:
+                    pytest.fail(f"Filter {name!r} didn't warn when setting custom attr")
+
+        assert '__dict__' not in BaseFilter.__slots__ if py_ver < (3, 7) else True, 'dict in abc'
+
+        class CustomFilter(MessageFilter):
+            def filter(self, message: Message):
+                pass
+
+        with pytest.warns(None):
+            CustomFilter().custom = 'allowed'  # Test setting custom attr to custom filters
+
+        with pytest.warns(TelegramDeprecationWarning, match='custom attributes'):
+            Filters().custom = 'raise warning'
+
     def test_filters_all(self, update):
         assert Filters.all(update)
 
@@ -131,19 +196,19 @@ class TestFilters:
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.regex('deep') | Filters.regex(r'linked param'))(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.regex('not int') | Filters.regex(r'linked param'))(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.regex('not int') & Filters.regex(r'linked param'))(update)
         assert not result
 
@@ -156,19 +221,19 @@ class TestFilters:
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.regex(r'linked param') & Filters.command)(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.regex(r'linked param') | Filters.command)(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         # Should not give a match since it's a or filter and it short circuits
         result = (Filters.command | Filters.regex(r'linked param'))(update)
         assert result is True
@@ -176,91 +241,91 @@ class TestFilters:
     def test_regex_complex_merges(self, update):
         SRE_TYPE = type(re.match("", ""))
         update.message.text = 'test it out'
-        filter = Filters.regex('test') & (
+        test_filter = Filters.regex('test') & (
             (Filters.status_update | Filters.forwarded) | Filters.regex('out')
         )
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
         assert len(matches) == 2
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.forward_date = datetime.datetime.utcnow()
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.text = 'test it'
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.forward_date = None
-        result = filter(update)
+        result = test_filter(update)
         assert not result
         update.message.text = 'test it out'
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.pinned_message = True
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.text = 'it out'
-        result = filter(update)
+        result = test_filter(update)
         assert not result
 
         update.message.text = 'test it out'
         update.message.forward_date = None
         update.message.pinned_message = None
-        filter = (Filters.regex('test') | Filters.command) & (
+        test_filter = (Filters.regex('test') | Filters.command) & (
             Filters.regex('it') | Filters.status_update
         )
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
         assert len(matches) == 2
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.text = 'test'
-        result = filter(update)
+        result = test_filter(update)
         assert not result
         update.message.pinned_message = True
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
         assert len(matches) == 1
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.text = 'nothing'
-        result = filter(update)
+        result = test_filter(update)
         assert not result
         update.message.text = '/start'
         update.message.entities = [MessageEntity(MessageEntity.BOT_COMMAND, 0, 6)]
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, bool)
         update.message.text = '/start it'
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
         assert len(matches) == 1
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
 
     def test_regex_inverted(self, update):
         update.message.text = '/start deep-linked param'
@@ -334,13 +399,13 @@ class TestFilters:
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.caption_regex('deep') | Filters.caption_regex(r'linked param'))(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.caption_regex('not int') | Filters.caption_regex(r'linked param'))(
             update
         )
@@ -348,7 +413,7 @@ class TestFilters:
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.caption_regex('not int') & Filters.caption_regex(r'linked param'))(
             update
         )
@@ -363,19 +428,19 @@ class TestFilters:
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.caption_regex(r'linked param') & Filters.command)(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         result = (Filters.caption_regex(r'linked param') | Filters.command)(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         # Should not give a match since it's a or filter and it short circuits
         result = (Filters.command | Filters.caption_regex(r'linked param'))(update)
         assert result is True
@@ -383,130 +448,130 @@ class TestFilters:
     def test_caption_regex_complex_merges(self, update):
         SRE_TYPE = type(re.match("", ""))
         update.message.caption = 'test it out'
-        filter = Filters.caption_regex('test') & (
+        test_filter = Filters.caption_regex('test') & (
             (Filters.status_update | Filters.forwarded) | Filters.caption_regex('out')
         )
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
         assert len(matches) == 2
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.forward_date = datetime.datetime.utcnow()
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.caption = 'test it'
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.forward_date = None
-        result = filter(update)
+        result = test_filter(update)
         assert not result
         update.message.caption = 'test it out'
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.pinned_message = True
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.caption = 'it out'
-        result = filter(update)
+        result = test_filter(update)
         assert not result
 
         update.message.caption = 'test it out'
         update.message.forward_date = None
         update.message.pinned_message = None
-        filter = (Filters.caption_regex('test') | Filters.command) & (
+        test_filter = (Filters.caption_regex('test') | Filters.command) & (
             Filters.caption_regex('it') | Filters.status_update
         )
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
         assert len(matches) == 2
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.caption = 'test'
-        result = filter(update)
+        result = test_filter(update)
         assert not result
         update.message.pinned_message = True
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
         assert len(matches) == 1
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
         update.message.caption = 'nothing'
-        result = filter(update)
+        result = test_filter(update)
         assert not result
         update.message.caption = '/start'
         update.message.entities = [MessageEntity(MessageEntity.BOT_COMMAND, 0, 6)]
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, bool)
         update.message.caption = '/start it'
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, dict)
         matches = result['matches']
         assert isinstance(matches, list)
         assert len(matches) == 1
-        assert all([type(res) == SRE_TYPE for res in matches])
+        assert all(type(res) is SRE_TYPE for res in matches)
 
     def test_caption_regex_inverted(self, update):
         update.message.caption = '/start deep-linked param'
         update.message.entities = [MessageEntity(MessageEntity.BOT_COMMAND, 0, 5)]
-        filter = ~Filters.caption_regex(r'deep-linked param')
-        result = filter(update)
+        test_filter = ~Filters.caption_regex(r'deep-linked param')
+        result = test_filter(update)
         assert not result
         update.message.caption = 'not it'
-        result = filter(update)
+        result = test_filter(update)
         assert result
         assert isinstance(result, bool)
 
-        filter = ~Filters.caption_regex('linked') & Filters.command
+        test_filter = ~Filters.caption_regex('linked') & Filters.command
         update.message.caption = "it's linked"
-        result = filter(update)
+        result = test_filter(update)
         assert not result
         update.message.caption = '/start'
         update.message.entities = [MessageEntity(MessageEntity.BOT_COMMAND, 0, 6)]
-        result = filter(update)
+        result = test_filter(update)
         assert result
         update.message.caption = '/linked'
-        result = filter(update)
+        result = test_filter(update)
         assert not result
 
-        filter = ~Filters.caption_regex('linked') | Filters.command
+        test_filter = ~Filters.caption_regex('linked') | Filters.command
         update.message.caption = "it's linked"
         update.message.entities = []
-        result = filter(update)
+        result = test_filter(update)
         assert not result
         update.message.caption = '/start linked'
         update.message.entities = [MessageEntity(MessageEntity.BOT_COMMAND, 0, 6)]
-        result = filter(update)
+        result = test_filter(update)
         assert result
         update.message.caption = '/start'
-        result = filter(update)
+        result = test_filter(update)
         assert result
         update.message.caption = 'nothig'
         update.message.entities = []
-        result = filter(update)
+        result = test_filter(update)
         assert result
 
     def test_filters_reply(self, update):
@@ -828,6 +893,11 @@ class TestFilters:
         assert Filters.status_update.chat_created(update)
         update.message.channel_chat_created = False
 
+        update.message.message_auto_delete_timer_changed = True
+        assert Filters.status_update(update)
+        assert Filters.status_update.message_auto_delete_timer_changed(update)
+        update.message.message_auto_delete_timer_changed = False
+
         update.message.migrate_to_chat_id = 100
         assert Filters.status_update(update)
         assert Filters.status_update.migrate(update)
@@ -853,6 +923,26 @@ class TestFilters:
         assert Filters.status_update.proximity_alert_triggered(update)
         update.message.proximity_alert_triggered = None
 
+        update.message.voice_chat_scheduled = 'scheduled'
+        assert Filters.status_update(update)
+        assert Filters.status_update.voice_chat_scheduled(update)
+        update.message.voice_chat_scheduled = None
+
+        update.message.voice_chat_started = 'hello'
+        assert Filters.status_update(update)
+        assert Filters.status_update.voice_chat_started(update)
+        update.message.voice_chat_started = None
+
+        update.message.voice_chat_ended = 'bye'
+        assert Filters.status_update(update)
+        assert Filters.status_update.voice_chat_ended(update)
+        update.message.voice_chat_ended = None
+
+        update.message.voice_chat_participants_invited = 'invited'
+        assert Filters.status_update(update)
+        assert Filters.status_update.voice_chat_participants_invited(update)
+        update.message.voice_chat_participants_invited = None
+
     def test_filters_forwarded(self, update):
         assert not Filters.forwarded(update)
         update.message.forward_date = datetime.datetime.utcnow()
@@ -1216,6 +1306,255 @@ class TestFilters:
         with pytest.raises(RuntimeError, match='Cannot set name'):
             f.name = 'foo'
 
+    def test_filters_forwarded_from_init(self):
+        with pytest.raises(RuntimeError, match='in conjunction with'):
+            Filters.forwarded_from(chat_id=1, username='chat')
+
+    def test_filters_forwarded_from_allow_empty(self, update):
+        assert not Filters.forwarded_from()(update)
+        assert Filters.forwarded_from(allow_empty=True)(update)
+
+    def test_filters_forwarded_from_id(self, update):
+        # Test with User id-
+        assert not Filters.forwarded_from(chat_id=1)(update)
+        update.message.forward_from.id = 1
+        assert Filters.forwarded_from(chat_id=1)(update)
+        update.message.forward_from.id = 2
+        assert Filters.forwarded_from(chat_id=[1, 2])(update)
+        assert not Filters.forwarded_from(chat_id=[3, 4])(update)
+        update.message.forward_from = None
+        assert not Filters.forwarded_from(chat_id=[3, 4])(update)
+
+        # Test with Chat id-
+        update.message.forward_from_chat.id = 4
+        assert Filters.forwarded_from(chat_id=[4])(update)
+        assert Filters.forwarded_from(chat_id=[3, 4])(update)
+
+        update.message.forward_from_chat.id = 2
+        assert not Filters.forwarded_from(chat_id=[3, 4])(update)
+        assert Filters.forwarded_from(chat_id=2)(update)
+
+    def test_filters_forwarded_from_username(self, update):
+        # For User username
+        assert not Filters.forwarded_from(username='chat')(update)
+        assert not Filters.forwarded_from(username='Testchat')(update)
+        update.message.forward_from.username = 'chat@'
+        assert Filters.forwarded_from(username='@chat@')(update)
+        assert Filters.forwarded_from(username='chat@')(update)
+        assert Filters.forwarded_from(username=['chat1', 'chat@', 'chat2'])(update)
+        assert not Filters.forwarded_from(username=['@username', '@chat_2'])(update)
+        update.message.forward_from = None
+        assert not Filters.forwarded_from(username=['@username', '@chat_2'])(update)
+
+        # For Chat username
+        assert not Filters.forwarded_from(username='chat')(update)
+        assert not Filters.forwarded_from(username='Testchat')(update)
+        update.message.forward_from_chat.username = 'chat@'
+        assert Filters.forwarded_from(username='@chat@')(update)
+        assert Filters.forwarded_from(username='chat@')(update)
+        assert Filters.forwarded_from(username=['chat1', 'chat@', 'chat2'])(update)
+        assert not Filters.forwarded_from(username=['@username', '@chat_2'])(update)
+        update.message.forward_from_chat = None
+        assert not Filters.forwarded_from(username=['@username', '@chat_2'])(update)
+
+    def test_filters_forwarded_from_change_id(self, update):
+        f = Filters.forwarded_from(chat_id=1)
+        # For User ids-
+        assert f.chat_ids == {1}
+        update.message.forward_from.id = 1
+        assert f(update)
+        update.message.forward_from.id = 2
+        assert not f(update)
+        f.chat_ids = 2
+        assert f.chat_ids == {2}
+        assert f(update)
+
+        # For Chat ids-
+        f = Filters.forwarded_from(chat_id=1)  # reset this
+        update.message.forward_from = None  # and change this to None, only one of them can be True
+        assert f.chat_ids == {1}
+        update.message.forward_from_chat.id = 1
+        assert f(update)
+        update.message.forward_from_chat.id = 2
+        assert not f(update)
+        f.chat_ids = 2
+        assert f.chat_ids == {2}
+        assert f(update)
+
+        with pytest.raises(RuntimeError, match='username in conjunction'):
+            f.usernames = 'chat'
+
+    def test_filters_forwarded_from_change_username(self, update):
+        # For User usernames
+        f = Filters.forwarded_from(username='chat')
+        update.message.forward_from.username = 'chat'
+        assert f(update)
+        update.message.forward_from.username = 'User'
+        assert not f(update)
+        f.usernames = 'User'
+        assert f(update)
+
+        # For Chat usernames
+        update.message.forward_from = None
+        f = Filters.forwarded_from(username='chat')
+        update.message.forward_from_chat.username = 'chat'
+        assert f(update)
+        update.message.forward_from_chat.username = 'User'
+        assert not f(update)
+        f.usernames = 'User'
+        assert f(update)
+
+        with pytest.raises(RuntimeError, match='chat_id in conjunction'):
+            f.chat_ids = 1
+
+    def test_filters_forwarded_from_add_chat_by_name(self, update):
+        chats = ['chat_a', 'chat_b', 'chat_c']
+        f = Filters.forwarded_from()
+
+        # For User usernames
+        for chat in chats:
+            update.message.forward_from.username = chat
+            assert not f(update)
+
+        f.add_usernames('chat_a')
+        f.add_usernames(['chat_b', 'chat_c'])
+
+        for chat in chats:
+            update.message.forward_from.username = chat
+            assert f(update)
+
+        # For Chat usernames
+        update.message.forward_from = None
+        f = Filters.forwarded_from()
+        for chat in chats:
+            update.message.forward_from_chat.username = chat
+            assert not f(update)
+
+        f.add_usernames('chat_a')
+        f.add_usernames(['chat_b', 'chat_c'])
+
+        for chat in chats:
+            update.message.forward_from_chat.username = chat
+            assert f(update)
+
+        with pytest.raises(RuntimeError, match='chat_id in conjunction'):
+            f.add_chat_ids(1)
+
+    def test_filters_forwarded_from_add_chat_by_id(self, update):
+        chats = [1, 2, 3]
+        f = Filters.forwarded_from()
+
+        # For User ids
+        for chat in chats:
+            update.message.forward_from.id = chat
+            assert not f(update)
+
+        f.add_chat_ids(1)
+        f.add_chat_ids([2, 3])
+
+        for chat in chats:
+            update.message.forward_from.username = chat
+            assert f(update)
+
+        # For Chat ids-
+        update.message.forward_from = None
+        f = Filters.forwarded_from()
+        for chat in chats:
+            update.message.forward_from_chat.id = chat
+            assert not f(update)
+
+        f.add_chat_ids(1)
+        f.add_chat_ids([2, 3])
+
+        for chat in chats:
+            update.message.forward_from_chat.username = chat
+            assert f(update)
+
+        with pytest.raises(RuntimeError, match='username in conjunction'):
+            f.add_usernames('chat')
+
+    def test_filters_forwarded_from_remove_chat_by_name(self, update):
+        chats = ['chat_a', 'chat_b', 'chat_c']
+        f = Filters.forwarded_from(username=chats)
+
+        with pytest.raises(RuntimeError, match='chat_id in conjunction'):
+            f.remove_chat_ids(1)
+
+        # For User usernames
+        for chat in chats:
+            update.message.forward_from.username = chat
+            assert f(update)
+
+        f.remove_usernames('chat_a')
+        f.remove_usernames(['chat_b', 'chat_c'])
+
+        for chat in chats:
+            update.message.forward_from.username = chat
+            assert not f(update)
+
+        # For Chat usernames
+        update.message.forward_from = None
+        f = Filters.forwarded_from(username=chats)
+        for chat in chats:
+            update.message.forward_from_chat.username = chat
+            assert f(update)
+
+        f.remove_usernames('chat_a')
+        f.remove_usernames(['chat_b', 'chat_c'])
+
+        for chat in chats:
+            update.message.forward_from_chat.username = chat
+            assert not f(update)
+
+    def test_filters_forwarded_from_remove_chat_by_id(self, update):
+        chats = [1, 2, 3]
+        f = Filters.forwarded_from(chat_id=chats)
+
+        with pytest.raises(RuntimeError, match='username in conjunction'):
+            f.remove_usernames('chat')
+
+        # For User ids
+        for chat in chats:
+            update.message.forward_from.id = chat
+            assert f(update)
+
+        f.remove_chat_ids(1)
+        f.remove_chat_ids([2, 3])
+
+        for chat in chats:
+            update.message.forward_from.username = chat
+            assert not f(update)
+
+        # For Chat ids
+        update.message.forward_from = None
+        f = Filters.forwarded_from(chat_id=chats)
+        for chat in chats:
+            update.message.forward_from_chat.id = chat
+            assert f(update)
+
+        f.remove_chat_ids(1)
+        f.remove_chat_ids([2, 3])
+
+        for chat in chats:
+            update.message.forward_from_chat.username = chat
+            assert not f(update)
+
+    def test_filters_forwarded_from_repr(self):
+        f = Filters.forwarded_from([1, 2])
+        assert str(f) == 'Filters.forwarded_from(1, 2)'
+        f.remove_chat_ids(1)
+        f.remove_chat_ids(2)
+        assert str(f) == 'Filters.forwarded_from()'
+        f.add_usernames('@foobar')
+        assert str(f) == 'Filters.forwarded_from(foobar)'
+        f.add_usernames('@barfoo')
+        assert str(f).startswith('Filters.forwarded_from(')
+        # we don't know the exact order
+        assert 'barfoo' in str(f) and 'foobar' in str(f)
+
+        with pytest.raises(RuntimeError, match='Cannot set name'):
+            f.name = 'foo'
+
     def test_filters_sender_chat_init(self):
         with pytest.raises(RuntimeError, match='in conjunction with'):
             Filters.sender_chat(chat_id=1, username='chat')
@@ -1379,6 +1718,16 @@ class TestFilters:
         update.message.sender_chat = None
         assert not Filters.sender_chat.channel(update)
 
+    def test_filters_is_automatic_forward(self, update):
+        assert not Filters.is_automatic_forward(update)
+        update.message.is_automatic_forward = True
+        assert Filters.is_automatic_forward(update)
+
+    def test_filters_has_protected_content(self, update):
+        assert not Filters.has_protected_content(update)
+        update.message.has_protected_content = True
+        assert Filters.has_protected_content(update)
+
     def test_filters_invoice(self, update):
         assert not Filters.invoice(update)
         update.message.invoice = 'test'
@@ -1453,6 +1802,13 @@ class TestFilters:
         assert not Filters.dice.darts(update)
         assert not Filters.dice.slot_machine([4])(update)
 
+        update.message.dice = Dice(5, '🎳')
+        assert Filters.dice.bowling(update)
+        assert Filters.dice.bowling([4, 5])(update)
+        assert not Filters.dice.dice(update)
+        assert not Filters.dice.darts(update)
+        assert not Filters.dice.bowling([4])(update)
+
     def test_language_filter_single(self, update):
         update.message.from_user.language_code = 'en_US'
         assert (Filters.language('en_US'))(update)
@@ -1890,3 +2246,18 @@ class TestFilters:
 
         with pytest.raises(RuntimeError, match='Cannot set name'):
             f.name = 'foo'
+
+    def test_filters_attachment(self, update):
+        assert not Filters.attachment(update)
+        # we need to define a new Update (or rather, message class) here because
+        # effective_attachment is only evaluated once per instance, and the filter relies on that
+        up = Update(
+            0,
+            Message(
+                0,
+                datetime.datetime.utcnow(),
+                Chat(0, 'private'),
+                document=Document("str", "other_str"),
+            ),
+        )
+        assert Filters.attachment(up)
