@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -25,15 +25,27 @@ from telegram import ForceReply, ReplyKeyboardRemove
 
 @pytest.fixture(scope='class')
 def force_reply():
-    return ForceReply(TestForceReply.force_reply, TestForceReply.selective)
+    return ForceReply(
+        TestForceReply.force_reply,
+        TestForceReply.selective,
+        TestForceReply.input_field_placeholder,
+    )
 
 
 class TestForceReply:
     force_reply = True
     selective = True
+    input_field_placeholder = 'force replies can be annoying if not used properly'
+
+    def test_slot_behaviour(self, force_reply, recwarn, mro_slots):
+        for attr in force_reply.__slots__:
+            assert getattr(force_reply, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not force_reply.__dict__, f"got missing slot(s): {force_reply.__dict__}"
+        assert len(mro_slots(force_reply)) == len(set(mro_slots(force_reply))), "duplicate slot"
+        force_reply.custom, force_reply.force_reply = 'should give warning', self.force_reply
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_message_with_force_reply(self, bot, chat_id, force_reply):
         message = bot.send_message(chat_id, 'text', reply_markup=force_reply)
 
@@ -42,6 +54,7 @@ class TestForceReply:
     def test_expected(self, force_reply):
         assert force_reply.force_reply == self.force_reply
         assert force_reply.selective == self.selective
+        assert force_reply.input_field_placeholder == self.input_field_placeholder
 
     def test_to_dict(self, force_reply):
         force_reply_dict = force_reply.to_dict()
@@ -49,6 +62,7 @@ class TestForceReply:
         assert isinstance(force_reply_dict, dict)
         assert force_reply_dict['force_reply'] == force_reply.force_reply
         assert force_reply_dict['selective'] == force_reply.selective
+        assert force_reply_dict['input_field_placeholder'] == force_reply.input_field_placeholder
 
     def test_equality(self):
         a = ForceReply(True, False)
