@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -272,7 +272,7 @@ class TestHelpers:
         test_message.text = None
 
         test_message = build_test_message(
-            sticker=Sticker('sticker_id', 'unique_id', 50, 50, False)
+            sticker=Sticker('sticker_id', 'unique_id', 50, 50, False, False)
         )
         assert helpers.effective_message_type(test_message) == 'sticker'
         test_message.sticker = None
