@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -127,6 +127,15 @@ class TestInputMediaVideo:
     supports_streaming = True
     caption_entities = [MessageEntity(MessageEntity.BOLD, 0, 2)]
 
+    def test_slot_behaviour(self, input_media_video, recwarn, mro_slots):
+        inst = input_media_video
+        for attr in inst.__slots__:
+            assert getattr(inst, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not inst.__dict__, f"got missing slot(s): {inst.__dict__}"
+        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"
+        inst.custom, inst.type = 'should give warning', self.type_
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_expected_values(self, input_media_video):
         assert input_media_video.type == self.type_
         assert input_media_video.media == self.media
@@ -185,6 +194,15 @@ class TestInputMediaPhoto:
     parse_mode = 'Markdown'
     caption_entities = [MessageEntity(MessageEntity.BOLD, 0, 2)]
 
+    def test_slot_behaviour(self, input_media_photo, recwarn, mro_slots):
+        inst = input_media_photo
+        for attr in inst.__slots__:
+            assert getattr(inst, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not inst.__dict__, f"got missing slot(s): {inst.__dict__}"
+        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"
+        inst.custom, inst.type = 'should give warning', self.type_
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_expected_values(self, input_media_photo):
         assert input_media_photo.type == self.type_
         assert input_media_photo.media == self.media
@@ -231,6 +249,15 @@ class TestInputMediaAnimation:
     height = 30
     duration = 1
 
+    def test_slot_behaviour(self, input_media_animation, recwarn, mro_slots):
+        inst = input_media_animation
+        for attr in inst.__slots__:
+            assert getattr(inst, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not inst.__dict__, f"got missing slot(s): {inst.__dict__}"
+        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"
+        inst.custom, inst.type = 'should give warning', self.type_
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_expected_values(self, input_media_animation):
         assert input_media_animation.type == self.type_
         assert input_media_animation.media == self.media
@@ -284,6 +311,15 @@ class TestInputMediaAudio:
     parse_mode = 'HTML'
     caption_entities = [MessageEntity(MessageEntity.BOLD, 0, 2)]
 
+    def test_slot_behaviour(self, input_media_audio, recwarn, mro_slots):
+        inst = input_media_audio
+        for attr in inst.__slots__:
+            assert getattr(inst, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not inst.__dict__, f"got missing slot(s): {inst.__dict__}"
+        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"
+        inst.custom, inst.type = 'should give warning', self.type_
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_expected_values(self, input_media_audio):
         assert input_media_audio.type == self.type_
         assert input_media_audio.media == self.media
@@ -341,6 +377,15 @@ class TestInputMediaDocument:
     caption_entities = [MessageEntity(MessageEntity.BOLD, 0, 2)]
     disable_content_type_detection = True
 
+    def test_slot_behaviour(self, input_media_document, recwarn, mro_slots):
+        inst = input_media_document
+        for attr in inst.__slots__:
+            assert getattr(inst, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not inst.__dict__, f"got missing slot(s): {inst.__dict__}"
+        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"
+        inst.custom, inst.type = 'should give warning', self.type_
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_expected_values(self, input_media_document):
         assert input_media_document.type == self.type_
         assert input_media_document.media == self.media
@@ -402,36 +447,38 @@ def media_group(photo, thumb):  # noqa: F811
 
 class TestSendMediaGroup:
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_media_group_photo(self, bot, chat_id, media_group):
         messages = bot.send_media_group(chat_id, media_group)
         assert isinstance(messages, list)
         assert len(messages) == 3
-        assert all([isinstance(mes, Message) for mes in messages])
-        assert all([mes.media_group_id == messages[0].media_group_id for mes in messages])
+        assert all(isinstance(mes, Message) for mes in messages)
+        assert all(mes.media_group_id == messages[0].media_group_id for mes in messages)
         assert all(mes.caption == f'photo {idx+1}' for idx, mes in enumerate(messages))
         assert all(
             mes.caption_entities == [MessageEntity(MessageEntity.BOLD, 0, 5)] for mes in messages
         )
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_media_group_all_args(self, bot, chat_id, media_group):
         m1 = bot.send_message(chat_id, text="test")
         messages = bot.send_media_group(
-            chat_id, media_group, disable_notification=True, reply_to_message_id=m1.message_id
+            chat_id,
+            media_group,
+            disable_notification=True,
+            reply_to_message_id=m1.message_id,
+            protect_content=True,
         )
         assert isinstance(messages, list)
         assert len(messages) == 3
-        assert all([isinstance(mes, Message) for mes in messages])
-        assert all([mes.media_group_id == messages[0].media_group_id for mes in messages])
+        assert all(isinstance(mes, Message) for mes in messages)
+        assert all(mes.media_group_id == messages[0].media_group_id for mes in messages)
         assert all(mes.caption == f'photo {idx+1}' for idx, mes in enumerate(messages))
         assert all(
             mes.caption_entities == [MessageEntity(MessageEntity.BOLD, 0, 5)] for mes in messages
         )
+        assert all(mes.has_protected_content for mes in messages)
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_media_group_custom_filename(
         self,
         bot,
@@ -474,7 +521,6 @@ class TestSendMediaGroup:
             bot.send_media_group(chat_id, [input_video, input_video])
 
     @flaky(3, 1)  # noqa: F811
-    @pytest.mark.timeout(10)  # noqa: F811
     def test_send_media_group_new_files(
         self, bot, chat_id, video_file, photo_file, animation_file  # noqa: F811
     ):  # noqa: F811
@@ -495,11 +541,10 @@ class TestSendMediaGroup:
 
         assert isinstance(messages, list)
         assert len(messages) == 3
-        assert all([isinstance(mes, Message) for mes in messages])
-        assert all([mes.media_group_id == messages[0].media_group_id for mes in messages])
+        assert all(isinstance(mes, Message) for mes in messages)
+        assert all(mes.media_group_id == messages[0].media_group_id for mes in messages)
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize(
         'default_bot,custom',
         [
@@ -534,7 +579,6 @@ class TestSendMediaGroup:
                 )
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_edit_message_media(self, bot, chat_id, media_group):
         messages = bot.send_media_group(chat_id, media_group)
         cid = messages[-1].chat.id
@@ -543,7 +587,6 @@ class TestSendMediaGroup:
         assert isinstance(new_message, Message)
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_edit_message_media_new_file(self, bot, chat_id, media_group, thumb_file):
         messages = bot.send_media_group(chat_id, media_group)
         cid = messages[-1].chat.id
@@ -553,8 +596,22 @@ class TestSendMediaGroup:
         )
         assert isinstance(new_message, Message)
 
+    def test_edit_message_media_with_thumb(
+        self, bot, chat_id, video_file, photo_file, monkeypatch  # noqa: F811
+    ):
+        def test(*args, **kwargs):
+            data = kwargs['fields']
+            video_check = data[input_video.media.attach] == input_video.media.field_tuple
+            thumb_check = data[input_video.thumb.attach] == input_video.thumb.field_tuple
+            result = video_check and thumb_check
+            raise Exception(f"Test was {'successful' if result else 'failing'}")
+
+        monkeypatch.setattr('telegram.utils.request.Request._request_wrapper', test)
+        input_video = InputMediaVideo(video_file, thumb=photo_file)
+        with pytest.raises(Exception, match='Test was successful'):
+            bot.edit_message_media(chat_id=chat_id, message_id=123, media=input_video)
+
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize(
         'default_bot', [{'parse_mode': ParseMode.HTML}], indirect=True, ids=['HTML-Bot']
     )
