@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -48,6 +48,11 @@ from telegram import (
     Dice,
     Bot,
     ChatAction,
+    VoiceChatStarted,
+    VoiceChatEnded,
+    VoiceChatParticipantsInvited,
+    MessageAutoDeleteTimerChanged,
+    VoiceChatScheduled,
 )
 from telegram.ext import Defaults
 from tests.conftest import check_shortcut_signature, check_shortcut_call, check_defaults_handling
@@ -100,7 +105,7 @@ def message(bot):
             )
         },
         {'photo': [PhotoSize('photo_id', 'unique_id', 50, 50)], 'caption': 'photo_file'},
-        {'sticker': Sticker('sticker_id', 'unique_id', 50, 50, True)},
+        {'sticker': Sticker('sticker_id', 'unique_id', 50, 50, True, False)},
         {'video': Video('video_id', 'unique_id', 12, 12, 12), 'caption': 'video_file'},
         {'voice': Voice('voice_id', 'unique_id', 5)},
         {'video_note': VideoNote('video_note_id', 'unique_id', 20, 12)},
@@ -115,6 +120,7 @@ def message(bot):
         {'group_chat_created': True},
         {'supergroup_chat_created': True},
         {'channel_chat_created': True},
+        {'message_auto_delete_timer_changed': MessageAutoDeleteTimerChanged(42)},
         {'migrate_to_chat_id': -12345},
         {'migrate_from_chat_id': -54321},
         {'pinned_message': Message(7, None, None, None)},
@@ -158,7 +164,6 @@ def message(bot):
                 ]
             },
         },
-        {'quote': True},
         {'dice': Dice(4, '🎲')},
         {'via_bot': User(9, 'A_Bot', True)},
         {
@@ -166,7 +171,17 @@ def message(bot):
                 User(1, 'John', False), User(2, 'Doe', False), 42
             )
         },
+        {'voice_chat_scheduled': VoiceChatScheduled(datetime.utcnow())},
+        {'voice_chat_started': VoiceChatStarted()},
+        {'voice_chat_ended': VoiceChatEnded(100)},
+        {
+            'voice_chat_participants_invited': VoiceChatParticipantsInvited(
+                [User(1, 'Rem', False), User(2, 'Emilia', False)]
+            )
+        },
         {'sender_chat': Chat(-123, 'discussion_channel')},
+        {'is_automatic_forward': True},
+        {'has_protected_content': True},
     ],
     ids=[
         'forwarded_user',
@@ -195,6 +210,7 @@ def message(bot):
         'group_created',
         'supergroup_created',
         'channel_created',
+        'message_auto_delete_timer_changed',
         'migrated_to',
         'migrated_from',
         'pinned',
@@ -207,11 +223,16 @@ def message(bot):
         'passport_data',
         'poll',
         'reply_markup',
-        'default_quote',
         'dice',
         'via_bot',
         'proximity_alert_triggered',
+        'voice_chat_scheduled',
+        'voice_chat_started',
+        'voice_chat_ended',
+        'voice_chat_participants_invited',
         'sender_chat',
+        'is_automatic_forward',
+        'has_protected_content',
     ],
 )
 def message_params(bot, request):
@@ -264,10 +285,11 @@ class TestMessage:
         {'length': 29, 'offset': 91, 'type': 'bold'},
         {'length': 9, 'offset': 101, 'type': 'strikethrough'},
         {'length': 10, 'offset': 129, 'type': 'pre', 'language': 'python'},
+        {'length': 7, 'offset': 141, 'type': 'spoiler'},
     ]
     test_text_v2 = (
         r'Test for <bold, ita_lic, \`code, links, text-mention and `\pre. '
-        'http://google.com and bold nested in strk>trgh nested in italic. Python pre.'
+        'http://google.com and bold nested in strk>trgh nested in italic. Python pre. Spoiled.'
     )
     test_message = Message(
         message_id=1,
@@ -290,6 +312,14 @@ class TestMessage:
         caption_entities=[MessageEntity(**e) for e in test_entities_v2],
     )
 
+    def test_slot_behaviour(self, message, recwarn, mro_slots):
+        for attr in message.__slots__:
+            assert getattr(message, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not message.__dict__, f"got missing slot(s): {message.__dict__}"
+        assert len(mro_slots(message)) == len(set(mro_slots(message))), "duplicate slot"
+        message.custom, message.message_id = 'should give warning', self.id_
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_all_possibilities_de_json_and_to_dict(self, bot, message_params):
         new = Message.de_json(message_params.to_dict(), bot)
 
@@ -359,7 +389,8 @@ class TestMessage:
             '<a href="tg://user?id=123456789">text-mention</a> and '
             r'<pre>`\pre</pre>. http://google.com '
             'and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. '
-            '<pre><code class="python">Python pre</code></pre>.'
+            '<pre><code class="python">Python pre</code></pre>. '
+            '<span class="tg-spoiler">Spoiled</span>.'
         )
         text_html = self.test_message_v2.text_html
         assert text_html == test_html_string
@@ -377,7 +408,8 @@ class TestMessage:
             '<a href="tg://user?id=123456789">text-mention</a> and '
             r'<pre>`\pre</pre>. <a href="http://google.com">http://google.com</a> '
             'and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. '
-            '<pre><code class="python">Python pre</code></pre>.'
+            '<pre><code class="python">Python pre</code></pre>. '
+            '<span class="tg-spoiler">Spoiled</span>.'
         )
         text_html = self.test_message_v2.text_html_urled
         assert text_html == test_html_string
@@ -398,7 +430,7 @@ class TestMessage:
             '[links](http://github.com/abc\\\\\\)def), '
             '[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. '
             r'http://google\.com and _bold *nested in ~strk\>trgh~ nested in* italic_\. '
-            '```python\nPython pre```\\.'
+            '```python\nPython pre```\\. ||Spoiled||\\.'
         )
         text_markdown = self.test_message_v2.text_markdown_v2
         assert text_markdown == test_md_string
@@ -420,6 +452,10 @@ class TestMessage:
         with pytest.raises(ValueError):
             message.text_markdown
 
+        message.entities = [MessageEntity(MessageEntity.SPOILER, offset=0, length=4)]
+        with pytest.raises(ValueError):
+            message.text_markdown
+
         message.entities = []
 
     def test_text_markdown_empty(self, message):
@@ -444,7 +480,7 @@ class TestMessage:
             '[links](http://github.com/abc\\\\\\)def), '
             '[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. '
             r'[http://google\.com](http://google.com) and _bold *nested in ~strk\>trgh~ '
-            'nested in* italic_\\. ```python\nPython pre```\\.'
+            'nested in* italic_\\. ```python\nPython pre```\\. ||Spoiled||\\.'
         )
         text_markdown = self.test_message_v2.text_markdown_v2_urled
         assert text_markdown == test_md_string
@@ -475,7 +511,8 @@ class TestMessage:
             '<a href="tg://user?id=123456789">text-mention</a> and '
             r'<pre>`\pre</pre>. http://google.com '
             'and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. '
-            '<pre><code class="python">Python pre</code></pre>.'
+            '<pre><code class="python">Python pre</code></pre>. '
+            '<span class="tg-spoiler">Spoiled</span>.'
         )
         caption_html = self.test_message_v2.caption_html
         assert caption_html == test_html_string
@@ -493,7 +530,8 @@ class TestMessage:
             '<a href="tg://user?id=123456789">text-mention</a> and '
             r'<pre>`\pre</pre>. <a href="http://google.com">http://google.com</a> '
             'and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. '
-            '<pre><code class="python">Python pre</code></pre>.'
+            '<pre><code class="python">Python pre</code></pre>. '
+            '<span class="tg-spoiler">Spoiled</span>.'
         )
         caption_html = self.test_message_v2.caption_html_urled
         assert caption_html == test_html_string
@@ -514,7 +552,7 @@ class TestMessage:
             '[links](http://github.com/abc\\\\\\)def), '
             '[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. '
             r'http://google\.com and _bold *nested in ~strk\>trgh~ nested in* italic_\. '
-            '```python\nPython pre```\\.'
+            '```python\nPython pre```\\. ||Spoiled||\\.'
         )
         caption_markdown = self.test_message_v2.caption_markdown_v2
         assert caption_markdown == test_md_string
@@ -541,7 +579,7 @@ class TestMessage:
             '[links](http://github.com/abc\\\\\\)def), '
             '[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. '
             r'[http://google\.com](http://google.com) and _bold *nested in ~strk\>trgh~ '
-            'nested in* italic_\\. ```python\nPython pre```\\.'
+            'nested in* italic_\\. ```python\nPython pre```\\. ||Spoiled||\\.'
         )
         caption_markdown = self.test_message_v2.caption_markdown_v2_urled
         assert caption_markdown == test_md_string
@@ -587,26 +625,26 @@ class TestMessage:
     def test_chat_id(self, message):
         assert message.chat_id == message.chat.id
 
-    @pytest.mark.parametrize('type', argvalues=[Chat.SUPERGROUP, Chat.CHANNEL])
-    def test_link_with_username(self, message, type):
+    @pytest.mark.parametrize('_type', argvalues=[Chat.SUPERGROUP, Chat.CHANNEL])
+    def test_link_with_username(self, message, _type):
         message.chat.username = 'username'
-        message.chat.type = type
+        message.chat.type = _type
         assert message.link == f'https://t.me/{message.chat.username}/{message.message_id}'
 
     @pytest.mark.parametrize(
-        'type, id', argvalues=[(Chat.CHANNEL, -1003), (Chat.SUPERGROUP, -1003)]
+        '_type, _id', argvalues=[(Chat.CHANNEL, -1003), (Chat.SUPERGROUP, -1003)]
     )
-    def test_link_with_id(self, message, type, id):
+    def test_link_with_id(self, message, _type, _id):
         message.chat.username = None
-        message.chat.id = id
-        message.chat.type = type
+        message.chat.id = _id
+        message.chat.type = _type
         # The leading - for group ids/ -100 for supergroup ids isn't supposed to be in the link
         assert message.link == f'https://t.me/c/{3}/{message.message_id}'
 
-    @pytest.mark.parametrize('id, username', argvalues=[(None, 'username'), (-3, None)])
-    def test_link_private_chats(self, message, id, username):
+    @pytest.mark.parametrize('_id, username', argvalues=[(None, 'username'), (-3, None)])
+    def test_link_private_chats(self, message, _id, username):
         message.chat.type = Chat.PRIVATE
-        message.chat.id = id
+        message.chat.id = _id
         message.chat.username = username
         assert message.link is None
         message.chat.type = Chat.GROUP
@@ -627,7 +665,6 @@ class TestMessage:
             'location',
             'venue',
             'invoice',
-            'invoice',
             'successful_payment',
         ):
             item = getattr(message_params, i, None)
@@ -698,7 +735,7 @@ class TestMessage:
             '[links](http://github.com/abc\\\\\\)def), '
             '[text\\-mention](tg://user?id=123456789) and ```\\`\\\\pre```\\. '
             r'http://google\.com and _bold *nested in ~strk\>trgh~ nested in* italic_\. '
-            '```python\nPython pre```\\.'
+            '```python\nPython pre```\\. ||Spoiled||\\.'
         )
 
         def make_assertion(*_, **kwargs):
@@ -737,7 +774,8 @@ class TestMessage:
             '<a href="tg://user?id=123456789">text-mention</a> and '
             r'<pre>`\pre</pre>. http://google.com '
             'and <i>bold <b>nested in <s>strk&gt;trgh</s> nested in</b> italic</i>. '
-            '<pre><code class="python">Python pre</code></pre>.'
+            '<pre><code class="python">Python pre</code></pre>. '
+            '<span class="tg-spoiler">Spoiled</span>.'
         )
 
         def make_assertion(*_, **kwargs):
@@ -1078,18 +1116,9 @@ class TestMessage:
             description = kwargs['description'] == 'description'
             payload = kwargs['payload'] == 'payload'
             provider_token = kwargs['provider_token'] == 'provider_token'
-            start_parameter = kwargs['start_parameter'] == 'start_parameter'
             currency = kwargs['currency'] == 'currency'
             prices = kwargs['prices'] == 'prices'
-            args = (
-                title
-                and description
-                and payload
-                and provider_token
-                and start_parameter
-                and currency
-                and prices
-            )
+            args = title and description and payload and provider_token and currency and prices
             return kwargs['chat_id'] == message.chat_id and args
 
         assert check_shortcut_signature(
@@ -1104,7 +1133,6 @@ class TestMessage:
             'description',
             'payload',
             'provider_token',
-            'start_parameter',
             'currency',
             'prices',
         )
@@ -1113,20 +1141,20 @@ class TestMessage:
             'description',
             'payload',
             'provider_token',
-            'start_parameter',
             'currency',
             'prices',
             quote=True,
         )
 
-    @pytest.mark.parametrize('disable_notification', [False, True])
-    def test_forward(self, monkeypatch, message, disable_notification):
+    @pytest.mark.parametrize('disable_notification,protected', [(False, True), (True, False)])
+    def test_forward(self, monkeypatch, message, disable_notification, protected):
         def make_assertion(*_, **kwargs):
             chat_id = kwargs['chat_id'] == 123456
             from_chat = kwargs['from_chat_id'] == message.chat_id
             message_id = kwargs['message_id'] == message.message_id
             notification = kwargs['disable_notification'] == disable_notification
-            return chat_id and from_chat and message_id and notification
+            protected_cont = kwargs['protect_content'] == protected
+            return chat_id and from_chat and message_id and notification and protected_cont
 
         assert check_shortcut_signature(
             Message.forward, Bot.forward_message, ['from_chat_id', 'message_id'], []
@@ -1135,11 +1163,13 @@ class TestMessage:
         assert check_defaults_handling(message.forward, message.bot)
 
         monkeypatch.setattr(message.bot, 'forward_message', make_assertion)
-        assert message.forward(123456, disable_notification=disable_notification)
+        assert message.forward(
+            123456, disable_notification=disable_notification, protect_content=protected
+        )
         assert not message.forward(635241)
 
-    @pytest.mark.parametrize('disable_notification', [True, False])
-    def test_copy(self, monkeypatch, message, disable_notification):
+    @pytest.mark.parametrize('disable_notification,protected', [(True, False), (False, True)])
+    def test_copy(self, monkeypatch, message, disable_notification, protected):
         keyboard = [[1, 2]]
 
         def make_assertion(*_, **kwargs):
@@ -1147,11 +1177,19 @@ class TestMessage:
             from_chat = kwargs['from_chat_id'] == message.chat_id
             message_id = kwargs['message_id'] == message.message_id
             notification = kwargs['disable_notification'] == disable_notification
+            protected_cont = kwargs['protect_content'] == protected
             if kwargs.get('reply_markup') is not None:
                 reply_markup = kwargs['reply_markup'] is keyboard
             else:
                 reply_markup = True
-            return chat_id and from_chat and message_id and notification and reply_markup
+            return (
+                chat_id
+                and from_chat
+                and message_id
+                and notification
+                and reply_markup
+                and protected_cont
+            )
 
         assert check_shortcut_signature(
             Message.copy, Bot.copy_message, ['from_chat_id', 'message_id'], []
@@ -1160,14 +1198,19 @@ class TestMessage:
         assert check_defaults_handling(message.copy, message.bot)
 
         monkeypatch.setattr(message.bot, 'copy_message', make_assertion)
-        assert message.copy(123456, disable_notification=disable_notification)
         assert message.copy(
-            123456, reply_markup=keyboard, disable_notification=disable_notification
+            123456, disable_notification=disable_notification, protect_content=protected
+        )
+        assert message.copy(
+            123456,
+            reply_markup=keyboard,
+            disable_notification=disable_notification,
+            protect_content=protected,
         )
         assert not message.copy(635241)
 
-    @pytest.mark.parametrize('disable_notification', [True, False])
-    def test_reply_copy(self, monkeypatch, message, disable_notification):
+    @pytest.mark.parametrize('disable_notification,protected', [(True, False), (False, True)])
+    def test_reply_copy(self, monkeypatch, message, disable_notification, protected):
         keyboard = [[1, 2]]
 
         def make_assertion(*_, **kwargs):
@@ -1175,6 +1218,7 @@ class TestMessage:
             from_chat = kwargs['chat_id'] == message.chat_id
             message_id = kwargs['message_id'] == 456789
             notification = kwargs['disable_notification'] == disable_notification
+            is_protected = kwargs['protect_content'] == protected
             if kwargs.get('reply_markup') is not None:
                 reply_markup = kwargs['reply_markup'] is keyboard
             else:
@@ -1183,7 +1227,15 @@ class TestMessage:
                 reply = kwargs['reply_to_message_id'] == message.message_id
             else:
                 reply = True
-            return chat_id and from_chat and message_id and notification and reply_markup and reply
+            return (
+                chat_id
+                and from_chat
+                and message_id
+                and notification
+                and reply_markup
+                and reply
+                and is_protected
+            )
 
         assert check_shortcut_signature(
             Message.reply_copy, Bot.copy_message, ['chat_id'], ['quote']
@@ -1192,12 +1244,22 @@ class TestMessage:
         assert check_defaults_handling(message.copy, message.bot)
 
         monkeypatch.setattr(message.bot, 'copy_message', make_assertion)
-        assert message.reply_copy(123456, 456789, disable_notification=disable_notification)
         assert message.reply_copy(
-            123456, 456789, reply_markup=keyboard, disable_notification=disable_notification
+            123456, 456789, disable_notification=disable_notification, protect_content=protected
         )
         assert message.reply_copy(
-            123456, 456789, quote=True, disable_notification=disable_notification
+            123456,
+            456789,
+            reply_markup=keyboard,
+            disable_notification=disable_notification,
+            protect_content=protected,
+        )
+        assert message.reply_copy(
+            123456,
+            456789,
+            quote=True,
+            disable_notification=disable_notification,
+            protect_content=protected,
         )
         assert message.reply_copy(
             123456,
@@ -1205,6 +1267,7 @@ class TestMessage:
             quote=True,
             reply_to_message_id=message.message_id,
             disable_notification=disable_notification,
+            protect_content=protected,
         )
 
     def test_edit_text(self, monkeypatch, message):
