@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -118,12 +118,16 @@ def check_object(h4):
         if field == 'from':
             field = 'from_user'
         elif (
-            name.startswith('InlineQueryResult') or name.startswith('InputMedia')
+            name.startswith('InlineQueryResult')
+            or name.startswith('InputMedia')
+            or name.startswith('BotCommandScope')
         ) and field == 'type':
             continue
-        elif name.startswith('PassportElementError') and field == 'source':
+        elif (name.startswith('ChatMember')) and field == 'status':
             continue
-        elif field == 'remove_keyboard':
+        elif (
+            name.startswith('PassportElementError') and field == 'source'
+        ) or field == 'remove_keyboard':
             continue
 
         param = sig.parameters.get(field)
@@ -135,8 +139,35 @@ def check_object(h4):
     ignored = IGNORED_PARAMETERS.copy()
     if name == 'InputFile':
         return
-    elif name == 'InlineQueryResult':
-        ignored |= {'id', 'type'}
+    if name == 'InlineQueryResult':
+        ignored |= {'id', 'type'}  # attributes common to all subclasses
+    if name == 'ChatMember':
+        ignored |= {'user', 'status'}  # attributes common to all subclasses
+    if name == 'ChatMember':
+        ignored |= {
+            'can_add_web_page_previews',  # for backwards compatibility
+            'can_be_edited',
+            'can_change_info',
+            'can_delete_messages',
+            'can_edit_messages',
+            'can_invite_users',
+            'can_manage_chat',
+            'can_manage_voice_chats',
+            'can_pin_messages',
+            'can_post_messages',
+            'can_promote_members',
+            'can_restrict_members',
+            'can_send_media_messages',
+            'can_send_messages',
+            'can_send_other_messages',
+            'can_send_polls',
+            'custom_title',
+            'is_anonymous',
+            'is_member',
+            'until_date',
+        }
+    if name == 'BotCommandScope':
+        ignored |= {'type'}  # attributes common to all subclasses
     elif name == 'User':
         ignored |= {'type'}  # TODO: Deprecation
     elif name in ('PassportFile', 'EncryptedPassportElement'):
