@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -54,6 +54,18 @@ def animated_sticker(bot, chat_id):
         return bot.send_sticker(chat_id, sticker=f, timeout=50).sticker
 
 
+@pytest.fixture(scope='function')
+def video_sticker_file():
+    with open('tests/data/telegram_video_sticker.webm', 'rb') as f:
+        yield f
+
+
+@pytest.fixture(scope='class')
+def video_sticker(bot, chat_id):
+    with open('tests/data/telegram_video_sticker.webm', 'rb') as f:
+        return bot.send_sticker(chat_id, sticker=f, timeout=50).sticker
+
+
 class TestSticker:
     # sticker_file_url = 'https://python-telegram-bot.org/static/testfiles/telegram.webp'
     # Serving sticker from gh since our server sends wrong content_type
@@ -66,6 +78,7 @@ class TestSticker:
     width = 510
     height = 512
     is_animated = False
+    is_video = False
     file_size = 39518
     thumb_width = 319
     thumb_height = 320
@@ -74,6 +87,14 @@ class TestSticker:
     sticker_file_id = '5a3128a4d2a04750b5b58397f3b5e812'
     sticker_file_unique_id = 'adc3145fd2e84d95b64d68eaa22aa33e'
 
+    def test_slot_behaviour(self, sticker, mro_slots, recwarn):
+        for attr in sticker.__slots__:
+            assert getattr(sticker, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not sticker.__dict__, f"got missing slot(s): {sticker.__dict__}"
+        assert len(mro_slots(sticker)) == len(set(mro_slots(sticker))), "duplicate slot"
+        sticker.custom, sticker.emoji = 'should give warning', self.emoji
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_creation(self, sticker):
         # Make sure file has been uploaded.
         assert isinstance(sticker, Sticker)
@@ -91,15 +112,17 @@ class TestSticker:
         assert sticker.width == self.width
         assert sticker.height == self.height
         assert sticker.is_animated == self.is_animated
+        assert sticker.is_video == self.is_video
         assert sticker.file_size == self.file_size
         assert sticker.thumb.width == self.thumb_width
         assert sticker.thumb.height == self.thumb_height
         assert sticker.thumb.file_size == self.thumb_file_size
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_all_args(self, bot, chat_id, sticker_file, sticker):
-        message = bot.send_sticker(chat_id, sticker=sticker_file, disable_notification=False)
+        message = bot.send_sticker(
+            chat_id, sticker=sticker_file, disable_notification=False, protect_content=True
+        )
 
         assert isinstance(message.sticker, Sticker)
         assert isinstance(message.sticker.file_id, str)
@@ -109,6 +132,7 @@ class TestSticker:
         assert message.sticker.width == sticker.width
         assert message.sticker.height == sticker.height
         assert message.sticker.is_animated == sticker.is_animated
+        assert message.sticker.is_video == sticker.is_video
         assert message.sticker.file_size == sticker.file_size
 
         assert isinstance(message.sticker.thumb, PhotoSize)
@@ -119,9 +143,9 @@ class TestSticker:
         assert message.sticker.thumb.width == sticker.thumb.width
         assert message.sticker.thumb.height == sticker.thumb.height
         assert message.sticker.thumb.file_size == sticker.thumb.file_size
+        assert message.has_protected_content
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_get_and_download(self, bot, sticker):
         new_file = bot.get_file(sticker.file_id)
 
@@ -135,14 +159,12 @@ class TestSticker:
         assert os.path.isfile('telegram.webp')
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_resend(self, bot, chat_id, sticker):
         message = bot.send_sticker(chat_id=chat_id, sticker=sticker.file_id)
 
         assert message.sticker == sticker
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_on_server_emoji(self, bot, chat_id):
         server_file_id = 'CAADAQADHAADyIsGAAFZfq1bphjqlgI'
         message = bot.send_sticker(chat_id=chat_id, sticker=server_file_id)
@@ -150,7 +172,6 @@ class TestSticker:
         assert sticker.emoji == self.emoji
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_from_url(self, bot, chat_id):
         message = bot.send_sticker(chat_id=chat_id, sticker=self.sticker_file_url)
         sticker = message.sticker
@@ -163,6 +184,7 @@ class TestSticker:
         assert message.sticker.width == sticker.width
         assert message.sticker.height == sticker.height
         assert message.sticker.is_animated == sticker.is_animated
+        assert message.sticker.is_video == sticker.is_video
         assert message.sticker.file_size == sticker.file_size
 
         assert isinstance(message.sticker.thumb, PhotoSize)
@@ -181,6 +203,7 @@ class TestSticker:
             'width': self.width,
             'height': self.height,
             'is_animated': self.is_animated,
+            'is_video': self.is_video,
             'thumb': sticker.thumb.to_dict(),
             'emoji': self.emoji,
             'file_size': self.file_size,
@@ -192,6 +215,7 @@ class TestSticker:
         assert json_sticker.width == self.width
         assert json_sticker.height == self.height
         assert json_sticker.is_animated == self.is_animated
+        assert json_sticker.is_video == self.is_video
         assert json_sticker.emoji == self.emoji
         assert json_sticker.file_size == self.file_size
         assert json_sticker.thumb == sticker.thumb
@@ -217,9 +241,9 @@ class TestSticker:
         monkeypatch.setattr(bot, '_post', make_assertion)
         bot.send_sticker(chat_id, file)
         assert test_flag
+        monkeypatch.delattr(bot, '_post')
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize(
         'default_bot,custom',
         [
@@ -262,17 +286,16 @@ class TestSticker:
         assert sticker_dict['width'] == sticker.width
         assert sticker_dict['height'] == sticker.height
         assert sticker_dict['is_animated'] == sticker.is_animated
+        assert sticker_dict['is_video'] == sticker.is_video
         assert sticker_dict['file_size'] == sticker.file_size
         assert sticker_dict['thumb'] == sticker.thumb.to_dict()
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_error_send_empty_file(self, bot, chat_id):
         with pytest.raises(TelegramError):
             bot.send_sticker(chat_id, open(os.devnull, 'rb'))
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_error_send_empty_file_id(self, bot, chat_id):
         with pytest.raises(TelegramError):
             bot.send_sticker(chat_id, '')
@@ -283,11 +306,18 @@ class TestSticker:
 
     def test_equality(self, sticker):
         a = Sticker(
-            sticker.file_id, sticker.file_unique_id, self.width, self.height, self.is_animated
+            sticker.file_id,
+            sticker.file_unique_id,
+            self.width,
+            self.height,
+            self.is_animated,
+            self.is_video,
         )
-        b = Sticker('', sticker.file_unique_id, self.width, self.height, self.is_animated)
-        c = Sticker(sticker.file_id, sticker.file_unique_id, 0, 0, False)
-        d = Sticker('', '', self.width, self.height, self.is_animated)
+        b = Sticker(
+            '', sticker.file_unique_id, self.width, self.height, self.is_animated, self.is_video
+        )
+        c = Sticker(sticker.file_id, sticker.file_unique_id, 0, 0, False, True)
+        d = Sticker('', '', self.width, self.height, self.is_animated, self.is_video)
         e = PhotoSize(
             sticker.file_id, sticker.file_unique_id, self.width, self.height, self.is_animated
         )
@@ -313,7 +343,9 @@ def sticker_set(bot):
         try:
             for i in range(1, 50):
                 bot.delete_sticker_from_set(ss.stickers[-i].file_id)
-        except BadRequest:
+        except BadRequest as e:
+            if e.message == 'Stickerset_not_modified':
+                return ss
             raise Exception('stickerset is growing too large.')
     return ss
 
@@ -325,7 +357,23 @@ def animated_sticker_set(bot):
         try:
             for i in range(1, 50):
                 bot.delete_sticker_from_set(ss.stickers[-i].file_id)
-        except BadRequest:
+        except BadRequest as e:
+            if e.message == 'Stickerset_not_modified':
+                return ss
+            raise Exception('stickerset is growing too large.')
+    return ss
+
+
+@pytest.fixture(scope='function')
+def video_sticker_set(bot):
+    ss = bot.get_sticker_set(f'video_test_by_{bot.username}')
+    if len(ss.stickers) > 100:
+        try:
+            for i in range(1, 50):
+                bot.delete_sticker_from_set(ss.stickers[-i].file_id)
+        except BadRequest as e:
+            if e.message == 'Stickerset_not_modified':
+                return ss
             raise Exception('stickerset is growing too large.')
     return ss
 
@@ -340,8 +388,9 @@ def sticker_set_thumb_file():
 class TestStickerSet:
     title = 'Test stickers'
     is_animated = True
+    is_video = True
     contains_masks = False
-    stickers = [Sticker('file_id', 'file_un_id', 512, 512, True)]
+    stickers = [Sticker('file_id', 'file_un_id', 512, 512, True, True)]
     name = 'NOTAREALNAME'
 
     def test_de_json(self, bot, sticker):
@@ -350,6 +399,7 @@ class TestStickerSet:
             'name': name,
             'title': self.title,
             'is_animated': self.is_animated,
+            'is_video': self.is_video,
             'contains_masks': self.contains_masks,
             'stickers': [x.to_dict() for x in self.stickers],
             'thumb': sticker.thumb.to_dict(),
@@ -359,15 +409,58 @@ class TestStickerSet:
         assert sticker_set.name == name
         assert sticker_set.title == self.title
         assert sticker_set.is_animated == self.is_animated
+        assert sticker_set.is_video == self.is_video
         assert sticker_set.contains_masks == self.contains_masks
         assert sticker_set.stickers == self.stickers
         assert sticker_set.thumb == sticker.thumb
 
+    def test_create_sticker_set(
+        self, bot, chat_id, sticker_file, animated_sticker_file, video_sticker_file
+    ):
+        """Creates the sticker set (if needed) which is required for tests. Make sure that this
+        test comes before the tests that actually use the sticker sets!
+        """
+        test_by = f"test_by_{bot.username}"
+        for sticker_set in [test_by, f'animated_{test_by}', f'video_{test_by}']:
+            try:
+                bot.get_sticker_set(sticker_set)
+            except BadRequest as e:
+                if not e.message == "Stickerset_invalid":
+                    raise e
+
+                if sticker_set.startswith(test_by):
+                    s = bot.create_new_sticker_set(
+                        chat_id,
+                        name=sticker_set,
+                        title="Sticker Test",
+                        png_sticker=sticker_file,
+                        emojis='😄',
+                    )
+                    assert s
+                elif sticker_set.startswith("animated"):
+                    a = bot.create_new_sticker_set(
+                        chat_id,
+                        name=sticker_set,
+                        title="Animated Test",
+                        tgs_sticker=animated_sticker_file,
+                        emojis='😄',
+                    )
+                    assert a
+                elif sticker_set.startswith("video"):
+                    v = bot.create_new_sticker_set(
+                        chat_id,
+                        name=sticker_set,
+                        title="Video Test",
+                        webm_sticker=video_sticker_file,
+                        emojis='🤔',
+                    )
+                    assert v
+
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_bot_methods_1_png(self, bot, chat_id, sticker_file):
         with open('tests/data/telegram_sticker.png', 'rb') as f:
-            file = bot.upload_sticker_file(95205500, f)
+            # chat_id was hardcoded as 95205500 but it stopped working for some reason
+            file = bot.upload_sticker_file(chat_id, f)
         assert file
         assert bot.add_sticker_to_set(
             chat_id, f'test_by_{bot.username}', png_sticker=file.file_id, emojis='😄'
@@ -382,7 +475,6 @@ class TestStickerSet:
         )
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_bot_methods_1_tgs(self, bot, chat_id):
         assert bot.add_sticker_to_set(
             chat_id,
@@ -391,6 +483,13 @@ class TestStickerSet:
             emojis='😄',
         )
 
+    @flaky(3, 1)
+    def test_bot_methods_1_webm(self, bot, chat_id):
+        with open('tests/data/telegram_video_sticker.webm', 'rb') as f:
+            assert bot.add_sticker_to_set(
+                chat_id, f'video_test_by_{bot.username}', webm_sticker=f, emojis='🤔'
+            )
+
     def test_sticker_set_to_dict(self, sticker_set):
         sticker_set_dict = sticker_set.to_dict()
 
@@ -398,23 +497,26 @@ class TestStickerSet:
         assert sticker_set_dict['name'] == sticker_set.name
         assert sticker_set_dict['title'] == sticker_set.title
         assert sticker_set_dict['is_animated'] == sticker_set.is_animated
+        assert sticker_set_dict['is_video'] == sticker_set.is_video
         assert sticker_set_dict['contains_masks'] == sticker_set.contains_masks
         assert sticker_set_dict['stickers'][0] == sticker_set.stickers[0].to_dict()
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_bot_methods_2_png(self, bot, sticker_set):
         file_id = sticker_set.stickers[0].file_id
         assert bot.set_sticker_position_in_set(file_id, 1)
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_bot_methods_2_tgs(self, bot, animated_sticker_set):
         file_id = animated_sticker_set.stickers[0].file_id
         assert bot.set_sticker_position_in_set(file_id, 1)
 
+    @flaky(3, 1)
+    def test_bot_methods_2_webm(self, bot, video_sticker_set):
+        file_id = video_sticker_set.stickers[0].file_id
+        assert bot.set_sticker_position_in_set(file_id, 1)
+
     @flaky(10, 1)
-    @pytest.mark.timeout(10)
     def test_bot_methods_3_png(self, bot, chat_id, sticker_set_thumb_file):
         sleep(1)
         assert bot.set_sticker_set_thumb(
@@ -422,30 +524,40 @@ class TestStickerSet:
         )
 
     @flaky(10, 1)
-    @pytest.mark.timeout(10)
     def test_bot_methods_3_tgs(self, bot, chat_id, animated_sticker_file, animated_sticker_set):
         sleep(1)
-        assert bot.set_sticker_set_thumb(
-            f'animated_test_by_{bot.username}', chat_id, animated_sticker_file
-        )
+        animated_test = f'animated_test_by_{bot.username}'
+        assert bot.set_sticker_set_thumb(animated_test, chat_id, animated_sticker_file)
         file_id = animated_sticker_set.stickers[-1].file_id
         # also test with file input and mask
-        assert bot.set_sticker_set_thumb(f'animated_test_by_{bot.username}', chat_id, file_id)
+        assert bot.set_sticker_set_thumb(animated_test, chat_id, file_id)
+
+    # TODO: Try the below by creating a custom .webm and not by downloading another pack's thumb
+    @pytest.mark.skip(
+        "Skipped for now since Telegram throws a 'File is too big' error "
+        "regardless of the .webm file size."
+    )
+    def test_bot_methods_3_webm(self, bot, chat_id, video_sticker_file, video_sticker_set):
+        pass
 
     @flaky(10, 1)
-    @pytest.mark.timeout(10)
     def test_bot_methods_4_png(self, bot, sticker_set):
         sleep(1)
         file_id = sticker_set.stickers[-1].file_id
         assert bot.delete_sticker_from_set(file_id)
 
     @flaky(10, 1)
-    @pytest.mark.timeout(10)
     def test_bot_methods_4_tgs(self, bot, animated_sticker_set):
         sleep(1)
         file_id = animated_sticker_set.stickers[-1].file_id
         assert bot.delete_sticker_from_set(file_id)
 
+    @flaky(10, 1)
+    def test_bot_methods_4_webm(self, bot, video_sticker_set):
+        sleep(1)
+        file_id = video_sticker_set.stickers[-1].file_id
+        assert bot.delete_sticker_from_set(file_id)
+
     def test_upload_sticker_file_local_files(self, monkeypatch, bot, chat_id):
         # For just test that the correct paths are passed as we have no local bot API set up
         test_flag = False
@@ -459,6 +571,7 @@ class TestStickerSet:
         monkeypatch.setattr(bot, '_post', make_assertion)
         bot.upload_sticker_file(chat_id, file)
         assert test_flag
+        monkeypatch.delattr(bot, '_post')
 
     def test_create_new_sticker_set_local_files(self, monkeypatch, bot, chat_id):
         # For just test that the correct paths are passed as we have no local bot API set up
@@ -468,13 +581,24 @@ class TestStickerSet:
 
         def make_assertion(_, data, *args, **kwargs):
             nonlocal test_flag
-            test_flag = data.get('png_sticker') == expected and data.get('tgs_sticker') == expected
+            test_flag = (
+                data.get('png_sticker') == expected
+                and data.get('tgs_sticker') == expected
+                and data.get('webm_sticker') == expected
+            )
 
         monkeypatch.setattr(bot, '_post', make_assertion)
         bot.create_new_sticker_set(
-            chat_id, 'name', 'title', 'emoji', png_sticker=file, tgs_sticker=file
+            chat_id,
+            'name',
+            'title',
+            'emoji',
+            png_sticker=file,
+            tgs_sticker=file,
+            webm_sticker=file,
         )
         assert test_flag
+        monkeypatch.delattr(bot, '_post')
 
     def test_add_sticker_to_set_local_files(self, monkeypatch, bot, chat_id):
         # For just test that the correct paths are passed as we have no local bot API set up
@@ -489,6 +613,7 @@ class TestStickerSet:
         monkeypatch.setattr(bot, '_post', make_assertion)
         bot.add_sticker_to_set(chat_id, 'name', 'emoji', png_sticker=file, tgs_sticker=file)
         assert test_flag
+        monkeypatch.delattr(bot, '_post')
 
     def test_set_sticker_set_thumb_local_files(self, monkeypatch, bot, chat_id):
         # For just test that the correct paths are passed as we have no local bot API set up
@@ -503,6 +628,7 @@ class TestStickerSet:
         monkeypatch.setattr(bot, '_post', make_assertion)
         bot.set_sticker_set_thumb('name', chat_id, thumb=file)
         assert test_flag
+        monkeypatch.delattr(bot, '_post')
 
     def test_get_file_instance_method(self, monkeypatch, sticker):
         def make_assertion(*_, **kwargs):
@@ -516,10 +642,26 @@ class TestStickerSet:
         assert sticker.get_file()
 
     def test_equality(self):
-        a = StickerSet(self.name, self.title, self.is_animated, self.contains_masks, self.stickers)
-        b = StickerSet(self.name, self.title, self.is_animated, self.contains_masks, self.stickers)
-        c = StickerSet(self.name, None, None, None, None)
-        d = StickerSet('blah', self.title, self.is_animated, self.contains_masks, self.stickers)
+        a = StickerSet(
+            self.name,
+            self.title,
+            self.is_animated,
+            self.contains_masks,
+            self.stickers,
+            self.is_video,
+        )
+        b = StickerSet(
+            self.name,
+            self.title,
+            self.is_animated,
+            self.contains_masks,
+            self.stickers,
+            self.is_video,
+        )
+        c = StickerSet(self.name, None, None, None, None, None)
+        d = StickerSet(
+            'blah', self.title, self.is_animated, self.contains_masks, self.stickers, self.is_video
+        )
         e = Audio(self.name, '', 0, None, None)
 
         assert a == b
