@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -18,7 +18,7 @@
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 import pytest
 
-from telegram import Update, User, Bot
+from telegram import Update, User, Bot, InlineKeyboardButton
 from telegram.utils.helpers import escape_markdown
 from tests.conftest import check_shortcut_signature, check_shortcut_call, check_defaults_handling
 
@@ -65,6 +65,14 @@ class TestUser:
     can_read_all_group_messages = True
     supports_inline_queries = False
 
+    def test_slot_behaviour(self, user, mro_slots, recwarn):
+        for attr in user.__slots__:
+            assert getattr(user, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not user.__dict__, f"got missing slot(s): {user.__dict__}"
+        assert len(mro_slots(user)) == len(set(mro_slots(user))), "duplicate slot"
+        user.custom, user.id = 'should give warning', self.id_
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_de_json(self, json_dict, bot):
         user = User.de_json(json_dict, bot)
 
@@ -287,18 +295,9 @@ class TestUser:
             description = kwargs['description'] == 'description'
             payload = kwargs['payload'] == 'payload'
             provider_token = kwargs['provider_token'] == 'provider_token'
-            start_parameter = kwargs['start_parameter'] == 'start_parameter'
             currency = kwargs['currency'] == 'currency'
             prices = kwargs['prices'] == 'prices'
-            args = (
-                title
-                and description
-                and payload
-                and provider_token
-                and start_parameter
-                and currency
-                and prices
-            )
+            args = title and description and payload and provider_token and currency and prices
             return kwargs['chat_id'] == user.id and args
 
         assert check_shortcut_signature(User.send_invoice, Bot.send_invoice, ['chat_id'], [])
@@ -311,7 +310,6 @@ class TestUser:
             'description',
             'payload',
             'provider_token',
-            'start_parameter',
             'currency',
             'prices',
         )
@@ -432,6 +430,40 @@ class TestUser:
         monkeypatch.setattr(user.bot, 'copy_message', make_assertion)
         assert user.copy_message(chat_id='chat_id', message_id='message_id')
 
+    def test_instance_method_approve_join_request(self, monkeypatch, user):
+        def make_assertion(*_, **kwargs):
+            chat_id = kwargs['chat_id'] == 'chat_id'
+            user_id = kwargs['user_id'] == user.id
+            return chat_id and user_id
+
+        assert check_shortcut_signature(
+            User.approve_join_request, Bot.approve_chat_join_request, ['user_id'], []
+        )
+        assert check_shortcut_call(
+            user.approve_join_request, user.bot, 'approve_chat_join_request'
+        )
+        assert check_defaults_handling(user.approve_join_request, user.bot)
+
+        monkeypatch.setattr(user.bot, 'approve_chat_join_request', make_assertion)
+        assert user.approve_join_request(chat_id='chat_id')
+
+    def test_instance_method_decline_join_request(self, monkeypatch, user):
+        def make_assertion(*_, **kwargs):
+            chat_id = kwargs['chat_id'] == 'chat_id'
+            user_id = kwargs['user_id'] == user.id
+            return chat_id and user_id
+
+        assert check_shortcut_signature(
+            User.decline_join_request, Bot.decline_chat_join_request, ['user_id'], []
+        )
+        assert check_shortcut_call(
+            user.decline_join_request, user.bot, 'decline_chat_join_request'
+        )
+        assert check_defaults_handling(user.decline_join_request, user.bot)
+
+        monkeypatch.setattr(user.bot, 'decline_chat_join_request', make_assertion)
+        assert user.decline_join_request(chat_id='chat_id')
+
     def test_mention_html(self, user):
         expected = '<a href="tg://user?id={}">{}</a>'
 
@@ -441,6 +473,13 @@ class TestUser:
         )
         assert user.mention_html(user.username) == expected.format(user.id, user.username)
 
+    def test_mention_button(self, user):
+        expected_name = InlineKeyboardButton(text="Bob", url=f"tg://user?id={user.id}")
+        expected_full = InlineKeyboardButton(text=user.full_name, url=f"tg://user?id={user.id}")
+
+        assert user.mention_button("Bob") == expected_name
+        assert user.mention_button() == expected_full
+
     def test_mention_markdown(self, user):
         expected = '[{}](tg://user?id={})'
 
