@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,6 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
-
 import pytest
 from flaky import flaky
 
@@ -46,6 +45,14 @@ class TestVenue:
     google_place_id = 'google place id'
     google_place_type = 'google place type'
 
+    def test_slot_behaviour(self, venue, mro_slots, recwarn):
+        for attr in venue.__slots__:
+            assert getattr(venue, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not venue.__dict__, f"got missing slot(s): {venue.__dict__}"
+        assert len(mro_slots(venue)) == len(set(mro_slots(venue))), "duplicate slot"
+        venue.custom, venue.title = 'should give warning', self.title
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_de_json(self, bot):
         json_dict = {
             'location': TestVenue.location.to_dict(),
@@ -84,7 +91,6 @@ class TestVenue:
         assert message
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize(
         'default_bot,custom',
         [
