@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -60,6 +60,14 @@ class TestVideo:
     video_file_id = '5a3128a4d2a04750b5b58397f3b5e812'
     video_file_unique_id = 'adc3145fd2e84d95b64d68eaa22aa33e'
 
+    def test_slot_behaviour(self, video, mro_slots, recwarn):
+        for attr in video.__slots__:
+            assert getattr(video, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not video.__dict__, f"got missing slot(s): {video.__dict__}"
+        assert len(mro_slots(video)) == len(set(mro_slots(video))), "duplicate slot"
+        video.custom, video.width = 'should give warning', self.width
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_creation(self, video):
         # Make sure file has been uploaded.
         assert isinstance(video, Video)
@@ -82,7 +90,6 @@ class TestVideo:
         assert video.mime_type == self.mime_type
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_all_args(self, bot, chat_id, video_file, video, thumb_file):
         message = bot.send_video(
             chat_id,
@@ -91,6 +98,7 @@ class TestVideo:
             caption=self.caption,
             supports_streaming=self.supports_streaming,
             disable_notification=False,
+            protect_content=True,
             width=video.width,
             height=video.height,
             parse_mode='Markdown',
@@ -114,9 +122,9 @@ class TestVideo:
         assert message.video.thumb.height == self.thumb_height
 
         assert message.video.file_name == self.file_name
+        assert message.has_protected_content
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_video_custom_filename(self, bot, chat_id, video_file, monkeypatch):
         def make_assertion(url, data, **kwargs):
             return data['video'].filename == 'custom_filename'
@@ -126,7 +134,6 @@ class TestVideo:
         assert bot.send_video(chat_id, video_file, filename='custom_filename')
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_get_and_download(self, bot, video):
         new_file = bot.get_file(video.file_id)
 
@@ -140,7 +147,6 @@ class TestVideo:
         assert os.path.isfile('telegram.mp4')
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_mp4_file_url(self, bot, chat_id, video):
         message = bot.send_video(chat_id, self.video_file_url, caption=self.caption)
 
@@ -166,7 +172,6 @@ class TestVideo:
         assert message.caption == self.caption
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_send_video_caption_entities(self, bot, chat_id, video):
         test_string = 'Italic Bold Code'
         entities = [
@@ -180,7 +185,6 @@ class TestVideo:
         assert message.caption_entities == entities
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_resend(self, bot, chat_id, video):
         message = bot.send_video(chat_id, video.file_id)
 
@@ -195,7 +199,6 @@ class TestVideo:
         assert message
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize('default_bot', [{'parse_mode': 'Markdown'}], indirect=True)
     def test_send_video_default_parse_mode_1(self, default_bot, chat_id, video):
         test_string = 'Italic Bold Code'
@@ -206,7 +209,6 @@ class TestVideo:
         assert message.caption == test_string
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize('default_bot', [{'parse_mode': 'Markdown'}], indirect=True)
     def test_send_video_default_parse_mode_2(self, default_bot, chat_id, video):
         test_markdown_string = '_Italic_ *Bold* `Code`'
@@ -218,7 +220,6 @@ class TestVideo:
         assert message.caption_markdown == escape_markdown(test_markdown_string)
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize('default_bot', [{'parse_mode': 'Markdown'}], indirect=True)
     def test_send_video_default_parse_mode_3(self, default_bot, chat_id, video):
         test_markdown_string = '_Italic_ *Bold* `Code`'
@@ -242,9 +243,9 @@ class TestVideo:
         monkeypatch.setattr(bot, '_post', make_assertion)
         bot.send_video(chat_id, file, thumb=file)
         assert test_flag
+        monkeypatch.delattr(bot, '_post')
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     @pytest.mark.parametrize(
         'default_bot,custom',
         [
@@ -314,13 +315,11 @@ class TestVideo:
         assert video_dict['file_name'] == video.file_name
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_error_send_empty_file(self, bot, chat_id):
         with pytest.raises(TelegramError):
             bot.send_video(chat_id, open(os.devnull, 'rb'))
 
     @flaky(3, 1)
-    @pytest.mark.timeout(10)
     def test_error_send_empty_file_id(self, bot, chat_id):
         with pytest.raises(TelegramError):
             bot.send_video(chat_id, '')
