@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2021
+# Copyright (C) 2015-2022
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,6 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
-
 import pytest
 import time
 
@@ -45,6 +44,14 @@ class TestWebhookInfo:
     max_connections = 42
     allowed_updates = ['type1', 'type2']
 
+    def test_slot_behaviour(self, webhook_info, mro_slots, recwarn):
+        for attr in webhook_info.__slots__:
+            assert getattr(webhook_info, attr, 'err') != 'err', f"got extra slot '{attr}'"
+        assert not webhook_info.__dict__, f"got missing slot(s): {webhook_info.__dict__}"
+        assert len(mro_slots(webhook_info)) == len(set(mro_slots(webhook_info))), "duplicate slot"
+        webhook_info.custom, webhook_info.url = 'should give warning', self.url
+        assert len(recwarn) == 1 and 'custom' in str(recwarn[0].message), recwarn.list
+
     def test_to_dict(self, webhook_info):
         webhook_info_dict = webhook_info.to_dict()
 
